#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# Copyright 2008 Glencoe Software, Inc. All rights reserved.
# Use is subject to license terms supplied in LICENSE.txt

"""
script plugin

Plugin read by omero.cli.Cli during initialization. The method(s)
defined here will be added to the Cli class for later use.

The script plugin is used to run arbitrary blitz scripts which
take as their sole input Ice configuration arguments, including
--Ice.Config=file1,file2.

The first parameter, the script itself, should be natively executable
on a given platform i.e. invokable by subprocess.call([file,...])
"""
from __future__ import print_function

from builtins import str
from builtins import object
import re
import os
import sys
import signal
import atexit

from omero.cli import CLI
from omero.cli import BaseControl

from omero.util.sessions import SessionsStore

from omero_ext.path import path

MIMETYPE = "text/x-python"

HELP = """Support for launching, uploading and otherwise managing \
OMERO.scripts"""

DEMO_SCRIPT = """#!/usr/bin/env python
import omero
import omero.rtypes as rtypes
import omero.scripts as scripts

o = scripts.Long("opt", min=0, max=5)
a = scripts.String("a", values=("foo", "bar"), optional=False)
b = scripts.Long("b").out()

client = scripts.client("length of input string",
\"\"\"
    Trivial example script which calculates the length
    of the string passed in as the "a" input, and returns
    the value as the long "b"
\"\"\", a, b, o,

authors = ["OME Team"],
institutions = ["openmicroscopy.org"])

print("Starting script")

try:
    a = client.getInput("a").getValue()
    b = len(a)
    client.setOutput("b", rtypes.rlong(b))
    client.setOutput("unregistered-output-param", rtypes.wrap([1,2,3]))
finally:
    client.closeSession()

print("Finished script")
"""

RE0 = re.compile(r"\s*script\s+upload\s*")
RE1 = re.compile(r"\s*script\s+upload\s+--official\s*")


class ScriptControl(BaseControl):

    def _complete(self, text, line, begidx, endidx):
        """
        Returns a file after "upload" and otherwise delegates to the
        BaseControl
        """
        for RE in (RE1, RE0):
            m = RE.match(line)
            if m:
                replaced = RE.sub('', line)
                suggestions = self._complete_file(replaced, os.getcwd())
                if False:  # line.find("--official") < 0:
                    add = "--official"
                    parts = line.split(" ")
                    if "--official".startswith(parts[-1]):
                        new = add[len(parts[-1]):]
                        if new:
                            add = new
                    suggestions.insert(0, add)
                return suggestions
        return BaseControl._complete(self, text, line, begidx, endidx)

    def _configure(self, parser):
        def _who(parser):
            return parser.add_argument(
                "who", nargs="*",
                help="Who to execute for: user, group, user=1, group=5"
                " (default=official)")

        parser.add_login_arguments()
        sub = parser.sub()

        # Disabling for 4.2 release. help = parser.add(sub, self.help,
        # "Extended help")

        demo = parser.add(
            sub, self.demo,
            "Runs a short demo of the scripting system")

        list = parser.add(
            sub, self.list, help="List files for user or group")
        _who(list)

        cat = parser.add(sub, self.cat, "Prints a script to standard out")
        edit = parser.add(
            sub, self.edit,
            "Opens a script in $EDITOR and saves it back to the server")
        params = parser.add(
            sub, self.params, help="Print the parameters for a given script")
        launch = parser.add(
            sub, self.launch, help="Launch a script with parameters")
        disable = parser.add(
            sub, self.disable,
            help="Makes script non-executable by setting the mimetype")
        disable.add_argument(
            "--mimetype", default="text/plain",
            help="Use a mimetype other than the default (%(default)s)")
        enable = parser.add(
            sub, self.enable, help="Makes a script executable (e.g. sets"
            " mimetype to %s)" % MIMETYPE)
        enable.add_argument(
            "--mimetype", default=MIMETYPE,
            help="Use a mimetype other than the default (%(default)s)")
        replace = parser.add(
            sub, self.replace,
            help="Replace an existing script with a new value")

        for x in (launch, params, cat, disable, enable, edit, replace):
            x.add_argument(
                "original_file",
                help="Id or path of a script file stored in OMERO")
        launch.add_argument(
            "input", nargs="*",
            help="Inputs for the script of the form 'param=value'")

        jobs = parser.add(
            sub, self.jobs, help="List current jobs for user or group")
        jobs.add_argument(
            "--all", action="store_true",
            help="Show all jobs, not just running ones")
        _who(jobs)

        serve = parser.add(
            sub, self.serve,
            help="Start a usermode processor for scripts")
        serve.add_argument(
            "--verbose", action="store_true",
            help="Enable debug logging on processor")
        serve.add_argument(
            "-b", "--background", action="store_true",
            help="Run processor in background. Used in demo")
        serve.add_argument(
            "-t", "--timeout", default=0, type=int,
            help="Seconds that the processor should run. 0 means no timeout")
        _who(serve)

        upload = parser.add(sub, self.upload, help="Upload a script")
        upload.add_argument(
            "--official", action="store_true",
            help="If set, creates a system script. Must be an admin")
        upload.add_argument(
            "file", help="Local script file to upload to OMERO")

        replace.add_argument(
            "file",
            help="Local script which should overwrite the existing one")

        delete = parser.add(
            sub, self.delete, help="delete an existing script")
        delete.add_argument(
            "id", type=int,
            help="Id of the original file which is to be deleted")

        run = parser.add(
            sub, self.run,
            help="Run a script with the OMERO libraries loaded and current"
            " login")
        run.add_argument("file", help="Local script file to run")
        run.add_argument(
            "input", nargs="*",
            help="Inputs for the script of the form 'param=value'")

        # log = parser.add(sub, self.log, help = "TBD", tbd="TRUE")
        for x in (demo, cat, edit, params, launch, disable, enable, jobs,
                  serve, upload, replace, delete, run):
            x.add_login_arguments()

    def help(self, args):
        self.ctx.out("""

        Available or planned(*) commands:
        ================================
        demo

        set
        set file=[id|name]
        set user=[id|name]
        set group=[id|name]
        set user=

        Note: Some of the other actions call set internally.


        cat
        cat file=[id|name]
        mv
        rm
        cp
        --replace / --overwrite

        register
        publish
        processors
        chain
        edit

        jobs
        jobs user
        jobs group

        launch
        launch file=[id|name]

        list
        list user
        list group
        list user=[id|name]
        list group=[id|name]
        list publication=[regex]
        list ofifical
        list namespace=[]

        log
        log file
        log user
        log group
        log file=[id|name]
        log user=[id|name]
        log group=[id|name]

        params
        params file=[id|name]

        serve
        serve --background
        serve --timeout={min}
        serve --verbose
        serve user
        serve group
        serve user=[id|name]
        serve group=[id|name]
        serve group=[id|name] user=[id|name]
        serve count=1
        serve log
        serve log=some/file/somewhere

        upload file=/tmp/my_script.py
        replace
        delete

        #
        # Other
        #

        run
        """)

    def demo(self, args):
        from omero.util.temp_files import create_path
        t = create_path("Demo_Script", ".py")

        try:
            from hashlib import sha1 as sha_new
        except ImportError:
            from sha import new as sha_new

        digest = sha_new()
        digest.update(DEMO_SCRIPT.encode('utf-8'))
        sha1 = digest.hexdigest()

        self.ctx.out("\nExample script writing session")
        self.ctx.out("="*80)

        def msg(title, method=None, *arguments):
            self.ctx.out("\n")
            self.ctx.out("\t+" + ("-"*68) + "+")
            title = "\t| %-66.66s | " % title
            self.ctx.out(title)
            if method:
                cmd = "%s %s" % (method.__name__, " ".join(arguments))
                cmd = "\t| COMMAND: omero script %-40.40s | " % cmd
                self.ctx.out(cmd)
            self.ctx.out("\t+" + ("-"*68) + "+")
            self.ctx.out(" ")
            if method:
                try:
                    self.ctx.invoke(['script', method.__name__] +
                                    list(arguments))
                except Exception as e:
                    import traceback
                    self.ctx.out("\nEXECUTION FAILED: %s" % e)
                    self.ctx.dbg(traceback.format_exc())

        client = self.ctx.conn(args)
        current_user = self.ctx.get_event_context().userId
        query = "select o from OriginalFile o where o.hash = '%s' and" \
            " o.details.owner.id = %s" % (sha1, current_user)
        files = client.sf.getQueryService().findAllByQuery(query, None)
        if len(files) == 0:
            msg("Saving demo script to %s" % t)
            t.write_text(DEMO_SCRIPT)

            msg("Uploading script", self.upload, str(t))
            id = self.ctx.get("script.file.id")
        else:
            id = files[0].id.val
            msg("Reusing demo script %s" % id)

        msg("Listing available scripts for user", self.list, "user")
        msg("Printing script content for file %s" % id, self.cat, str(id))
        msg("Serving file %s in background" % id, self.serve, "user",
            "--background")
        msg("Printing script params for file %s" % id, self.params,
            "file=%s" % id)
        msg("Launching script with parameters: a=bad-string (fails)",
            self.launch, "file=%s" % id, "a=bad-string")
        msg("Launching script with parameters: a=bad-string opt=6 (fails)",
            self.launch, "file=%s" % id, "a=bad-string", "opt=6")
        msg("Launching script with parameters: a=foo opt=1 (passes)",
            self.launch, "file=%s" % id, "a=foo", "opt=1")
        try:
            for p in list(getattr(self, "_processors", [])):
                p.cleanup()
                self._processors.remove(p)
        except Exception as e:
            self.ctx.err("Failed to clean processors: %s" % e)

        self.ctx.out("\nDeleting script from server...")
        args.id = int(id)
        self.delete(args)

    def cat(self, args):
        client = self.ctx.conn(args)
        script_id, ofile = self._file(args, client)
        try:
            self.ctx.out(client.sf.getScriptService().getScriptText(script_id))
        except Exception as e:
            self.ctx.err("Failed to find script: %s (%s)" % (script_id, e))

    def edit(self, args):
        client = self.ctx.conn(args)
        scriptSvc = client.sf.getScriptService()
        script_id, ofile = self._file(args, client)
        txt = None
        try:
            txt = client.sf.getScriptService().getScriptText(script_id)
            if not txt:
                self.ctx.err("No text for script: %s" % script_id)
                self.ctx.err("Does this file appear in the script list?")
                self.ctx.err("If not, try 'replace'")
                return
            from omero.util.temp_files import create_path
            from omero.util import edit_path
            p = create_path()
            edit_path(p, txt)
            scriptSvc.editScript(ofile, p.text())
        except Exception as e:
            self.ctx.err("Failed to find script: %s (%s)" % (script_id, e))

    def jobs(self, args):
        self.ctx.conn(args)
        cols = ("username", "groupname", "started", "finished")
        query = "select j, %s, s.value from Job j join j.status s" \
            % (",".join(["j.%s" % j for j in cols]))
        if not args.all:
            query += " where j.finished is null"

        self.ctx.out("Running query via 'hql' subcommand: %s" % query)
        self.ctx.invoke("""hql "%s" """ % query)

    def launch(self, args):
        """
        """

        client = self.ctx.conn(args)
        script_id, ofile = self._file(args, client)

        import omero
        import omero.scripts
        import omero.rtypes
        svc = client.sf.getScriptService()
        try:
            params = svc.getParams(script_id)
        except omero.ValidationException as ve:
            self.ctx.die(502, "ValidationException: %s" % ve.message)

        m = self._parse_inputs(args, params)

        try:
            proc = svc.runScript(script_id, m, None)
            job = proc.getJob()
        except omero.ValidationException as ve:
            self.ctx.err("Bad parameters:\n%s" % ve)
            return  # EARLY EXIT

        # Adding notification to wait on result
        cb = omero.scripts.ProcessCallbackI(client, proc)
        try:
            self.ctx.out("Job %s ready" % job.id.val)
            self.ctx.out("Waiting....")
            while proc.poll() is None:
                cb.block(1000)
            self.ctx.out("Callback received: %s" % cb.block(0))
            rv = proc.getResults(3)
        finally:
            cb.close()

        def p(m):
            class handle(object):
                def write(this, val):
                    val = "\t* %s" % val.decode()
                    val = val.replace("\n", "\n\t* ")
                    self.ctx.out(val, newline=False)

                def close(this):
                    pass

            f = rv.get(m, None)
            if f and f.val:
                self.ctx.out("\n\t*** start %s (id=%s)***"
                             % (m, f.val.id.val))
                try:
                    client.download(ofile=f.val, filehandle=handle())
                except Exception:
                    self.ctx.err("Failed to display %s" % m)
                self.ctx.out("\n\t*** end %s ***\n" % m)

        p("stdout")
        p("stderr")
        self.ctx.out("\n\t*** out parameters ***")
        for k, v in list(rv.items()):
            if k not in ("stdout", "stderr", "omero.scripts.parse"):
                self.ctx.out("\t* %s=%s" % (k, omero.rtypes.unwrap(v)))
        self.ctx.out("\t***  done ***")

    def list(self, args):
        client = self.ctx.conn(args)
        sf = client.sf
        svc = sf.getScriptService()
        if args.who:
            who = [self._parse_who(w) for w in args.who]
            scripts = svc.getUserScripts(who)
            banner = "Scripts for %s" % ", ".join(args.who)
        else:
            scripts = svc.getScripts()
            banner = "Official scripts"
        self._parse_scripts(scripts, banner)

    def log(self, args):
        print(args)
        pass

    def params(self, args):
        client = self.ctx.conn(args)
        script_id, ofile = self._file(args, client)
        import omero
        svc = client.sf.getScriptService()

        try:
            job_params = svc.getParams(script_id)
        except omero.ValidationException as ve:
            self.ctx.die(454, "ValidationException: %s" % ve.message)
        except omero.ResourceError as re:
            self.ctx.die(455, "ResourceError: %s" % re.message)

        if job_params:
            self.ctx.out("")
            self.ctx.out("id:  %s" % script_id)
            self.ctx.out("name:  %s" % job_params.name)
            self.ctx.out("version:  %s" % job_params.version)
            self.ctx.out("authors:  %s" % ", ".join(job_params.authors))
            self.ctx.out("institutions:  %s"
                         % ", ".join(job_params.institutions))
            self.ctx.out("description:  %s" % job_params.description)
            self.ctx.out("namespaces:  %s" % ", ".join(job_params.namespaces))
            self.ctx.out("stdout:  %s" % job_params.stdoutFormat)
            self.ctx.out("stderr:  %s" % job_params.stderrFormat)

            def print_params(which, params):
                import omero
                self.ctx.out(which)
                for k in sorted(params,
                                key=lambda name: params.get(name).grouping):
                    v = params.get(k)
                    self.ctx.out("  %s - %s" % (k, (v.description and
                                 v.description or "(no description)")))
                    self.ctx.out("    Optional: %s" % v.optional)
                    self.ctx.out("    Type: %s" % v.prototype.ice_staticId())
                    if isinstance(v.prototype, omero.RCollection):
                        coll = v.prototype.val
                        if len(coll) == 0:
                            self.ctx.out("    Subtype: (empty)")
                        else:
                            self.ctx.out("    Subtype: %s"
                                         % coll[0].ice_staticId())

                    elif isinstance(v.prototype, omero.RMap):
                        try:
                            proto_value = \
                                v.prototype.val.values[0].ice_staticId()
                        except Exception:
                            proto_value = None

                        self.ctx.out("    Subtype: %s" % proto_value)

                    # ticket:11472 - string min/max need quoting
                    def min_max(x):
                        if x:
                            if x.val is None:
                                return ""
                            elif isinstance(x, omero.RString):
                                return "'%s'" % x.val
                            else:
                                return x.val
                        return ""

                    self.ctx.out("    Min: %s" % min_max(v.min))
                    self.ctx.out("    Max: %s" % min_max(v.max))
                    values = omero.rtypes.unwrap(v.values)
                    self.ctx.out("    Values: %s"
                                 % (values and ", ".join(values) or ""))
            print_params("inputs:", job_params.inputs)
            print_params("outputs:", job_params.outputs)

    def serve(self, args):

        # List of processors which have been started
        if not hasattr(self, "_processors"):
            self._processors = []

        debug = args.verbose
        background = args.background
        timeout = args.timeout
        client = self.ctx.conn(args)
        who = [self._parse_who(w) for w in args.who]
        if not who:
            who = []  # Official scripts only

        # Similar to omero.util.Server starting here
        import logging
        original = list(logging._handlerList)
        roots = list(logging.getLogger().handlers)
        logging._handlerList = []
        logging.getLogger().handlers = []

        from omero.util import configure_logging
        from omero.processor import usermode_processor
        lvl = debug and 10 or 20
        configure_logging(loglevel=lvl)

        try:
            try:
                impl = usermode_processor(
                    client, serverid="omero.scripts.serve", accepts_list=who,
                    omero_home=self.ctx.dir)
                self._processors.append(impl)
            except Exception as e:
                self.ctx.die(100, "Failed initialization: %s" % e)

            if background:
                def cleanup():
                    impl.cleanup()
                    logging._handlerList = original
                    logging.getLogger().handlers = roots
                atexit.register(cleanup)
            else:
                if self._isWindows():
                    self.foreground_win(impl, timeout)
                else:
                    self.foreground_nix(impl, timeout)
        finally:
            if not background:
                logging._handlerList = original
                logging.getLogger().handlers = roots

        return impl

    def foreground_nix(self, impl, timeout):
        """
        Use signal.SIGALRM to wait for the timeout to signal
        """

        def handler(signum, frame):
            raise SystemExit()

        old = signal.signal(signal.SIGALRM, handler)
        try:
            signal.alarm(timeout)
            self.ctx.input("Press any key to exit...\n")
            signal.alarm(0)
        finally:
            self.ctx.dbg("DONE")
            signal.signal(signal.SIGTERM, old)
            impl.cleanup()

    def foreground_win(self, impl, timeout):
        """
        Note: currently simply fails.
        An implementation might be possible using msvcrt.
        See: \
http://stackoverflow.com/questions/3471461/raw-input-and-timeout/3911560
        """
        try:
            if timeout != 0:
                self.ctx.die(144, "Timeout not supported on Windows")
            else:
                self.ctx.input("Press any key to exit...\n")
                self.ctx.dbg("DONE")
        finally:
            impl.cleanup()

    def upload(self, args):
        p = path(args.file)
        if not p.exists():
            self.ctx.die(502, "File does not exist: %s" % p.abspath())

        import omero
        c = self.ctx.conn(args)
        scriptSvc = c.sf.getScriptService()

        if args.official:
            try:
                id = scriptSvc.uploadOfficialScript(args.file, p.text())
            except omero.ApiUsageException as aue:
                if "editScript" in aue.message:
                    self.ctx.die(502, "%s already exists; use 'replace'"
                                 " instead" % args.file)
                else:
                    self.ctx.die(504, "ApiUsageException: %s" % aue.message)
            except omero.SecurityViolation as sv:
                self.ctx.die(503, "SecurityViolation: %s" % sv.message)
        else:
            id = scriptSvc.uploadScript(args.file, p.text())

        self.ctx.err("Uploaded %sscript"
                     % (args.official and "official " or ""))
        self.ctx.out("OriginalFile:%s" % id)
        self.ctx.set("script.file.id", id)

    def replace(self, args):
        import omero
        client = self.ctx.conn(args)
        script_id, ofile = self._file(args, client)
        fpath = args.file

        file = open(fpath)
        scriptText = file.read()
        file.close()
        scriptSvc = client.sf.getScriptService()

        try:
            scriptSvc.editScript(ofile, scriptText)
        except omero.SecurityViolation as sv:
            self.ctx.die(200, sv.message)

    def delete(self, args):
        ofile = args.id
        client = self.ctx.conn(args)
        try:
            client.sf.getScriptService().deleteScript(ofile)
        except Exception as e:
            self.ctx.err("Failed to delete script: %s (%s)" % (ofile, e))

    def disable(self, args):
        ofile = self.setmimetype(args)
        self.ctx.out("Disabled %s by setting mimetype to %s"
                     % (ofile.id.val, args.mimetype))

    def enable(self, args):
        ofile = self.setmimetype(args)
        self.ctx.out("Enabled %s by setting mimetype to %s"
                     % (ofile.id.val, args.mimetype))

    def setmimetype(self, args):
        from omero.rtypes import rstring
        client = self.ctx.conn(args)
        script_id, ofile = self._file(args, client)
        if args.mimetype == MIMETYPE:  # is default
            if ofile.name.val.endswith(".jy"):
                args.mimetype = "text/x-jython"
            elif ofile.name.val.endswith(".m"):
                args.mimetype = "text/x-matlab"
        ofile.setMimetype(rstring(args.mimetype))
        return client.sf.getUpdateService().saveAndReturnObject(ofile)

    #
    # Other
    #
    def run(self, args):
        if not os.path.exists(args.file):
            self.ctx.die(670, "No such file: %s" % args.file)
        else:
            client = self.ctx.conn(args)
            store = SessionsStore()
            srv, usr, uuid, port = store.get_current()
            props = store.get(srv, usr, uuid)

            from omero.scripts import parse_file
            from omero.util.temp_files import create_path
            path = create_path()
            text = """
omero.host=%(omero.host)s
omero.user=%(omero.sess)s
omero.pass=%(omero.sess)s
            """
            path.write_text(text % props)

            params = parse_file(args.file)
            m = self._parse_inputs(args, params)
            for k, v in list(m.items()):
                if v is not None:
                    client.setInput(k, v)

            p = self.ctx.popen([sys.executable, args.file], stdout=sys.stdout,
                               stderr=sys.stderr, ICE_CONFIG=str(path))
            p.wait()
            if p.poll() != 0:
                self.ctx.die(p.poll(), "Execution failed.")

    #
    # Helpers
    #
    def _parse_inputs(self, args, params):
        from omero.scripts import parse_inputs, parse_input, MissingInputs
        try:
            rv = parse_inputs(args.input, params)
        except MissingInputs as mi:
            rv = mi.inputs
            for key in mi.keys:
                value = self.ctx.input("""Enter value for "%s": """ % key,
                                       required=True)
                rv.update(parse_input("%s=%s" % (key, value), params))
        return rv

    def _parse_scripts(self, scripts, msg):
        """
        Parses a list of scripts to self.ctx.out
        """
        from omero.util.text import TableBuilder
        tb = TableBuilder("id", msg)
        for x in scripts:
            tb.row(x.id.val, x.path.val + x.name.val)
        self.ctx.out(str(tb.build()))

    def _file(self, args, client):
        f = args.original_file
        q = client.sf.getQueryService()
        svc = client.sf.getScriptService()

        if f is None:
            self.ctx.die(100, "No script provided")
        elif f.startswith("file="):
            f = f[5:]

        try:
            script_id = int(f)
        except Exception:
            script_path = str(f)
            script_id = svc.getScriptID(script_path)
        ofile = q.get("OriginalFile", script_id)

        return script_id, ofile

    def _parse_who(self, who):
        """
        Parses who items of the form: "user", "group", "user=1", "group=6"
        """

        import omero
        WHO_FACTORY = {"user": omero.model.ExperimenterI,
                       "group": omero.model.ExperimenterGroupI}
        WHO_CURRENT = {"user": lambda ec: ec.userId,
                       "group": lambda ec: ec.groupId}

        for key, factory in list(WHO_FACTORY.items()):
            if who.startswith(key):
                if who == key:
                    id = WHO_CURRENT[key](self.ctx.get_event_context())
                    return factory(id, False)
                else:
                    parts = who.split("=")
                    if len(parts) != 2:
                        continue
                    else:
                        id = int(parts[1])
                        return factory(id, False)


try:
    register("script", ScriptControl, HELP)
except NameError:
    if __name__ == "__main__":
        cli = CLI()
        cli.register("script", ScriptControl, HELP)
        cli.invoke(sys.argv[1:])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# OMERO Utilities package
#
# Copyright 2009 Glencoe Software, Inc.  All Rights Reserved.
# Use is subject to license terms supplied in LICENSE.txt
#

from __future__ import division
from builtins import str
from future.utils import native_str
from past.utils import old_div
from builtins import object
from appdirs import user_data_dir, user_cache_dir
import os
import sys
import Ice
import shlex
import omero
import IcePy
import IceGrid
import logging
import platform
import Glacier2
import threading
import logging.handlers
import omero.util.concurrency
import uuid
import omero.ObjectFactoryRegistrar as ofr

from omero.util.decorators import locked
from omero_version import omero_version

import omero_ext.path as path

LOGDIR = os.path.join("var", "log")
LOGFORMAT = "%(asctime)s %(levelname)-5.5s [%(name)40s] " \
            "(%(threadName)-10s) %(message)s"
LOGLEVEL = logging.INFO
LOGSIZE = 500000000
LOGNUM = 9
LOGMODE = "a"

orig_stdout = sys.stdout
orig_stderr = sys.stderr

# Application name, Author (Windows only), Major version
APPDIR_DEFAULTS = ('OMERO.py', 'OME', omero_version.split('.')[0])


def make_logname(self):
    """
    Generates a logname from the given instance using the module
    and name from its class
    """
    log_name = "%s.%s" % (self.__class__.__module__, self.__class__.__name__)
    return log_name


def configure_logging(logdir=None, logfile=None, loglevel=LOGLEVEL,
                      format=LOGFORMAT, filemode=LOGMODE, maxBytes=LOGSIZE,
                      backupCount=LOGNUM, time_rollover=False):

    if logdir is None or logfile is None:
        handler = logging.StreamHandler()
    elif not time_rollover:
        handler = logging.handlers.RotatingFileHandler(
            os.path.join(logdir, logfile), maxBytes=maxBytes,
            backupCount=backupCount)
    else:
        handler = logging.handlers.TimedRotatingFileHandler(
            os.path.join(logdir, logfile), 'midnight', 1)
        # Windows will not allow renaming (or deleting) a file that's open.
        # There's nothing the logging package can do about that.
        try:
            sys.getwindowsversion()
        except:
            handler.doRollover()

    handler.setLevel(loglevel)
    formatter = logging.Formatter(format)
    handler.setFormatter(formatter)
    rootLogger = logging.getLogger()
    rootLogger.setLevel(loglevel)
    rootLogger.addHandler(handler)
    return rootLogger


def configure_server_logging(props):
    """
    Takes an Ice.Properties instance and configures logging
    """
    program_name = props.getProperty("Ice.Admin.ServerId")
    # Using Ice.ProgramName on Windows failed
    log_dir = props.getPropertyWithDefault("omero.logging.directory", LOGDIR)
    log_name = program_name + ".log"
    log_timed = props.getPropertyWithDefault(
        "omero.logging.timedlog", "False")[0] in ('T', 't')
    log_num = int(
        props.getPropertyWithDefault("omero.logging.lognum", native_str(LOGNUM)))
    log_size = int(
        props.getPropertyWithDefault("omero.logging.logsize", native_str(LOGSIZE)))
    log_num = int(
        props.getPropertyWithDefault("omero.logging.lognum", native_str(LOGNUM)))
    log_level = int(
        props.getPropertyWithDefault("omero.logging.level", native_str(LOGLEVEL)))
    configure_logging(log_dir, log_name, loglevel=log_level,
                      maxBytes=log_size, backupCount=log_num,
                      time_rollover=log_timed)

    sys.stdout = StreamRedirect(logging.getLogger("stdout"))
    sys.stderr = StreamRedirect(logging.getLogger("stderr"))


class StreamRedirect(object):

    """
    Since all server components should exclusively using the logging module
    any output to stdout or stderr is caught and logged at "WARN". This is
    useful, especially in the case of Windows, where stdout/stderr is eaten.
    """

    def __init__(self, logger):
        self.logger = logger
        self.internal = logging.getLogger("StreamRedirect")
        self.softspace = False

    def flush(self):
        pass

    def write(self, msg):
        msg = msg.strip()
        if msg:
            self.logger.warn(msg)

    def __getattr__(self, name):
        self.internal.warn("No attribute: %s" % name)


class Dependency(object):

    """
    Centralized logic for declaring and logging a service
    dependency on a non-shipped library. This is called
    lazily from the run method of the application to give
    logging time to be initialized.

    See #4566
    """

    def __init__(self, key):
        self.key = key

    def get_version(self, target):
        """
        Get version method which returns a string
        representing. Should be overwritten by
        subclasses for packages/modules with no
        __version__ field.
        """
        return target.__version__

    def check(self, logger):
        try:
            target = __import__(self.key)
            version = self.get_version(target)
            logger.info("Loaded dependency %s (%s)" % (self.key, version))
            return True
        except ImportError:
            logger.error("Failed to load: '%s'" % self.key)
            return False


def internal_service_factory(communicator, user="root", group=None, retries=6,
                             interval=10, client_uuid=None, stop_event=None):
    """
    Try to return a ServiceFactory from the grid.

    Try a number of times then give up and raise the
    last exception returned. This method will only
    work internally to the grid, i.e. behind the Glacier2
    firewall. It is intended for internal servers to
    be able to create sessions for accessing the database. ::

        communicator := Ice.Communicator used to find the registry
        user         := Username which should have a session created
        group        := Group into which the session should be logged
        retries      := Number of session creation retries before throwing
        interval     := Seconds between retries
        client_uuid  := Uuid of the client which should be used
    """
    log = logging.getLogger("omero.utils")
    if stop_event is None:
        stop_event = omero.util.concurrency.get_event(
            name="internal_service_factory")

    tryCount = 0
    excpt = None
    query = communicator.stringToProxy("IceGrid/Query")
    query = IceGrid.QueryPrx.checkedCast(query)

    implicit_ctx = communicator.getImplicitContext()
    implicit_ctx.put(omero.constants.AGENT, "Python service")
    if client_uuid is not None:
        implicit_ctx.put(omero.constants.CLIENTUUID, client_uuid)
    else:
        if not implicit_ctx.containsKey(omero.constants.CLIENTUUID):
            client_uuid = native_str(uuid.uuid4())
            implicit_ctx.put(omero.constants.CLIENTUUID, client_uuid)

    while tryCount < retries:
        if stop_event.isSet():  # Something is shutting down, exit.
            return None
        try:
            blitz = query.findAllObjectsByType("::Glacier2::SessionManager")[0]
            blitz = Glacier2.SessionManagerPrx.checkedCast(blitz)
            sf = blitz.create(user, None)
            # Group currently unused.
            return omero.api.ServiceFactoryPrx.checkedCast(sf)
        except Exception as e:
            tryCount += 1
            log.info("Failed to get session on attempt %s", str(tryCount))
            excpt = e
            stop_event.wait(interval)

    log.warn("Reason: %s", str(excpt))
    if excpt:
        raise excpt


def create_admin_session(communicator):
    """
    """
    reg = communicator.stringToProxy("IceGrid/Registry")
    reg = IceGrid.RegistryPrx.checkedCast(reg)
    adm = reg.createAdminSession('null', '')
    return adm


def add_grid_object(communicator, obj):
    """
    """
    communicator.identityToString(obj.ice_getIdentity())
    adm = create_admin_session(communicator)
    prx = adm.getAdmin()
    try:
        try:
            prx.addObject(obj)
        except IceGrid.ObjectExistsException:
            prx.updateObject(obj)
    finally:
        adm.destroy()


def long_to_path(id, root=""):
    """
    Converts a long to a path such that for all directiories only
    a 1000 files and a 1000 subdirectories will be returned.

    This method duplicates the logic in
    ome.io.nio.AbstractFileSystemService.java:getPath()
    """
    suffix = ""
    remaining = id
    dirno = 0

    if id is None or id == "":
        raise Exception("Expecting a not-null id.")

    id = int(id)

    if id < 0:
        raise Exception("Expecting a non-negative id.")

    while (remaining > 999):
        remaining /= 1000

        if remaining > 0:
            dirno = remaining % 1000
            suffix = os.path.join("Dir-%03d" % dirno, suffix)

    return os.path.join(root, "%s%s" % (suffix, id))


def load_dotted_class(dotted_class):
    """
    Load a Python class of the form "pkg.mod.Class"
    via __import__ and return it. No ctor or similar
    is called.
    """
    try:
        parts = dotted_class.split(".")
        pkg = ".".join(parts[0:-2])
        mod = native_str(parts[-2])
        kls = parts[-1]
        got = __import__(pkg, fromlist=[mod])
        got = getattr(got, mod)
        return getattr(got, kls)
    except Exception as e:
        raise Exception("""Failed to load: %s
        previous excetion: %s""" % (dotted_class, e))


class ServerContext(object):

    """
    Context passed to all servants.

    server_id, communicator, and stop_event will be
    constructed by the top-level Server instance.

    A context instance may also be configured to hold
    on to an internal session (ServiceFactoryPrx) and
    keep it alive.

    This instance obeys the Resources API and calls
    sf.keepAlive(None) on every check call, but does
    nothing on cleanup. The sf instance must be manually
    cleaned as the final operation of a servant.

    (Note: cleanup of the server context indicates
    server shutdown, so should be infrequent)
    """

    def __init__(self, server_id, communicator, stop_event,
                 on_newsession=None):
        self._lock = threading.RLock()
        self.logger = logging.getLogger("omero.util.ServerContext")
        self.server_id = server_id
        self.communicator = communicator
        self.stop_event = stop_event
        self.servant_map = dict()
        self.on_newsession = None

    @locked
    def add_servant(self, adapter_or_current, servant, ice_identity=None):
        oa = adapter_or_current
        if isinstance(adapter_or_current, (Ice.Current, IcePy.Current)):
            oa = oa.adapter
        if ice_identity is None:
            prx = oa.addWithUUID(servant)
        else:
            prx = oa.add(servant, ice_identity)

        servant.setProxy(prx)
        self.servant_map[prx] = servant
        return prx

    def newSession(self):
        self.session = internal_service_factory(
            self.communicator, stop_event=self.stop_event)
        if callable(self.on_newsession):
            self.on_newsession(self.session)

    def hasSession(self):
        return hasattr(self, "session")

    @locked
    def getSession(self, recreate=True):
        """
        Returns the ServiceFactoryPrx configured for the context if
        available. If the context was not configured for sessions,
        an ApiUsageException will be thrown: servants should know
        whether or not they were configured for sessions.
        See Servant(..., needs_session = True)

        Otherwise, if there is no ServiceFactoryPrx, an attempt will
        be made to create one if recreate == True. If the value is None
        or non can be recreated, an InternalException will be thrown.

        TODO : currently no arguments are provided for re-creating these,
        but also not in Servant.__init__
        """
        if not self.hasSession():
            raise omero.ApiUsageException(
                "Not configured for server connection")

        if self.session:
            try:
                self.session.keepAlive(None)
            except Ice.CommunicatorDestroyedException:
                self.session = None  # Ignore
            except Exception as e:
                self.logger.warn("Connection failure: %s" % e)
                self.session = None

        if self.session is None and recreate:
            try:
                self.newSession()
                self.logger.info("Established connection: %s" % self.session)
            except Exception as e:
                self.logger.warn("Failed to establish connection: %s" % e)

        if self.session is None:
            raise omero.InternalException("No connection to server")

        return self.session

    def check(self):
        """
        Calls getSession() but always returns True. This keeps the context
        available in the resources for later uses, and tries to re-establish
        a connection in case Blitz goes down.
        """
        try:
            self.getSession()
        except:
            pass
        return True

    def cleanup(self):
        """
        Does nothing. Context clean up must happen manually
        since later activities may want to reuse it. Servants using
        a server connection should cleanup the instance *after* Resources
        is cleaned up
        """
        pass


class Server(Ice.Application):

    """
    Basic server implementation which can be used for
    implementing a standalone python server which can
    be started from icegridnode.

    The servant implementation MUST have a constructor
    which takes a single ServerContext argument AND
    have a cleanup() method

    Logging is configured relative to the current directory
    to be in var/log by default.

    Usage::

        if __name__ == "__main__":
            app=Server(
                ServicesI, "ServicesAdapter", Ice.Identity("Services",""))
            sys.exit(app.main(sys.argv))

    app.impl now points to an instance of ServicesI

    """

    def __init__(self, impl_class, adapter_name, identity, logdir=LOGDIR,
                 dependencies=()):

        self.impl_class = impl_class
        self.adapter_name = adapter_name
        self.identity = identity
        self.logdir = logdir
        self.stop_event = omero.util.concurrency.get_event(name="Server")
        self.dependencies = dependencies

    def waitOnStartup(self):
        ms = 10000  # 10 seconds by default
        try:
            i = os.environ.get("OMERO_STARTUP_WAIT", "10000")
            ms = int(i)
        except:
            self.logger.debug(exc_info=1)

        try:
            self.logger.info("Waiting %s ms on startup" % ms)
            self.stop_event.wait(old_div(ms, 1000))
        except:
            self.logger.debug(exc_info=1)

    def run(self, args):

        from omero.rtypes import ObjectFactories as rFactories
        from omero.columns import ObjectFactories as cFactories

        props = self.communicator().getProperties()
        configure_server_logging(props)

        self.logger = logging.getLogger("omero.util.Server")
        self.logger.info("*" * 80)
        self.waitOnStartup()
        self.logger.info("Starting")

        failures = 0
        for x in self.dependencies:
            if not x.check(self.logger):
                failures += 1
        if failures:
            self.logger.error("Missing dependencies: %s" % failures)
            sys.exit(50)

        self.shutdownOnInterrupt()

        try:

            ofr.registerObjectFactory(self.communicator(), None)  # No client
            for of in list(rFactories.values()) + list(cFactories.values()):
                of.register(self.communicator())

            try:
                serverid = self.communicator().getProperties().getProperty(
                    "Ice.ServerId")
                ctx = ServerContext(
                    serverid, self.communicator(), self.stop_event)
                self.impl = self.impl_class(ctx)
                getattr(self.impl, "cleanup")  # Required per docs
            except:
                self.logger.error("Failed initialization", exc_info=1)
                sys.exit(100)

            try:
                self.adapter = self.communicator().createObjectAdapter(
                    self.adapter_name)
                self.adapter.activate()
                # calls setProxy
                ctx.add_servant(self.adapter, self.impl, self.identity)
                # ticket:1978 for non-collocated registries
                prx = self.adapter.createDirectProxy(self.identity)
                # This must happen _after_ activation
                add_grid_object(self.communicator(), prx)
            except:
                self.logger.error("Failed activation", exc_info=1)
                sys.exit(200)

            self.logger.info("Entering main loop")
            self.communicator().waitForShutdown()
        finally:
            self.stop_event.set()  # Let's all waits shutdown
            self.logger.info("Cleanup")
            self.cleanup()
            self.logger.info("Stopped")
            self.logger.info("*" * 80)

    def cleanup(self):
        """
        Cleans up all resources that were created by this server.
        Primarily the one servant instance.
        """
        if hasattr(self, "impl"):
            try:
                self.impl.cleanup()
            finally:
                del self.impl


class SimpleServant(object):

    """
    Base servant initialization. Doesn't create or try to cleanup
    a top-level Resources thread. This is useful for large numbers
    of servants. For servers and other singleton-like servants,
    see "Servant"
    """

    def __init__(self, ctx):
        self._lock = threading.RLock()
        self.prx = None  # Proxy which points to self
        self.ctx = ctx
        self.stop_event = ctx.stop_event
        self.communicator = ctx.communicator
        self.logger = logging.getLogger(make_logname(self))
        self.logger.debug("Created")

    def setProxy(self, prx):
        """
        Should be overwritten for post-initialization activities.
        The reason this method exists is that the implementation
        must be complete before registering it with the adapter.
        """
        self.prx = prx


class Servant(SimpleServant):

    """
    Abstract servant which can be used along with a slice2py
    generated dispatch class as the base type of high-level servants.
    These provide resource cleanup as per the omero.util.Server
    class.

    By passing "needs_session = True" to this constructor, an internal
    session will be created and stored in ServerContext as well as
    registered with self.resources
    """

    def __init__(self, ctx, needs_session=False):
        SimpleServant.__init__(self, ctx)
        self.resources = omero.util.Resources(
            sleeptime=60, stop_event=self.stop_event)
        if needs_session:
            self.ctx.newSession()
            self.resources.add(self.ctx)

    def cleanup(self):
        """
        Cleanups all resoures created by this servant. Calling
        cleanup multiple times should be safe.
        """
        resources = self.resources
        self.resources = None
        if resources is not None:
            self.logger.info("Cleaning up")
            resources.cleanup()
            self.logger.info("Done")
        if self.ctx.hasSession():
            try:
                sf = self.ctx.getSession(recreate=False)
                self.logger.debug("Destroying %s" % sf)
                sf.destroy()
            except:
                pass

    def __del__(self):
        self.cleanup()


class Resources(object):

    """
    Container class for storing resources which should be
    cleaned up on close and periodically checked. Use
    stop_event.set() to stop the internal thread.
    """

    def __init__(self, sleeptime=60, stop_event=None):
        """
        Add resources via add(object). They should have a no-arg cleanup()
        and a check() method.

        The check method will be called periodically (default: 60 seconds)
        on each resource. The cleanup method will be called on
        Resources.cleanup()
        """

        self.stuff = []
        self._lock = threading.RLock()
        self.logger = logging.getLogger("omero.util.Resources")
        self.stop_event = stop_event
        if not self.stop_event:
            self.stop_event = omero.util.concurrency.get_event(
                name="Resources")

        if sleeptime < 5:
            raise Exception(
                "Sleep time should be greater than 5: %s" % sleeptime)

        self.sleeptime = sleeptime

        class Task(threading.Thread):

            """
            Internal thread used for checking "stuff"
            """

            def run(self):
                ctx = self.ctx  # Outer class
                ctx.logger.info("Starting")
                while not ctx.stop_event.isSet():
                    try:
                        ctx.logger.debug("Executing")
                        copy = ctx.copyStuff()
                        remove = ctx.checkAll(copy)
                        ctx.removeAll(remove)
                    except:
                        ctx.logger.error(
                            "Exception during execution", exc_info=True)

                    ctx.logger.debug("Sleeping %s" % ctx.sleeptime)
                    # ticket:1531 - Attempting to catch threading issues
                    try:
                        ctx.stop_event.wait(ctx.sleeptime)
                    except ValueError:
                        pass

                if isinstance(ctx.stop_event,
                              omero.util.concurrency.AtExitEvent):
                    if ctx.stop_event.atexit:
                        return  # Skipping log. See #3260

                ctx.logger.info("Halted")

        self.thread = Task()
        self.thread.ctx = self
        self.thread.start()

    @locked
    def copyStuff(self):
        """
        Within a lock, copy the "stuff" list and reverse it.
        The list is reversed so that entries added
        later, which may depend on earlier added entries
        get a chance to be cleaned up first.
        """
        copy = list(self.stuff)
        copy.reverse()
        return copy

    # Not locked
    def checkAll(self, copy):
        """
        While stop_event is unset, go through the copy
        of stuff and call the check method on each
        entry. Any that throws an exception or returns
        a False value will be returned in the remove list.
        """
        remove = []
        for m in copy:
            if self.stop_event.isSet():
                return  # Let cleanup handle this
            self.logger.debug("Checking %s" % m[0])
            method = getattr(m[0], m[2])
            rv = None
            try:
                rv = method()
            except:
                self.logger.warn("Error from %s" % method, exc_info=True)
            if not rv:
                remove.append(m)
        return remove

    @locked
    def removeAll(self, remove):
        """
        Finally, within another lock, call the "cleanup"
        method on all the entries in remove, and remove
        them from the official stuff list. (If stop_event
        is set during execution, we return with the assumption
        that Resources.cleanup() will take care of them)
        """
        for r in remove:
            if self.stop_event.isSet():
                return  # Let cleanup handle this
            self.logger.debug("Removing %s" % r[0])
            self.safeClean(r)
            self.stuff.remove(r)

    @locked
    def add(self, object, cleanupMethod="cleanup", checkMethod="check"):
        entry = (object, cleanupMethod, checkMethod)
        self.logger.debug("Adding object %s" % object)
        self.stuff.append(entry)

    @locked
    def cleanup(self):
        stuff = self.stuff
        self.stuff = None
        self.stop_event.set()
        if stuff:
            for m in stuff:
                self.safeClean(m)
        self.logger.debug("Cleanup done")

    def safeClean(self, m):
        try:
            self.logger.debug("Cleaning %s" % m[0])
            method = getattr(m[0], m[1])
            method()
        except:
            self.logger.error("Error cleaning resource: %s" % m[0], exc_info=1)

    def __del__(self):
        self.cleanup()


class Environment(object):

    """
    Simple class for creating an executable environment
    """

    def __init__(self, *args):
        """
        Takes an number of environment variable names which
        should be copied to the target environment if present
        in the current execution environment.
        """
        if sys.platform == "win32":
            # Prevents SocketException. See ticket:1518
            self.env = os.environ.copy()
        else:
            self.env = {}
        for arg in args:
            if arg in os.environ:
                self.env[arg] = os.environ[arg]

    def __call__(self):
        """
        Returns the environment map when called.
        """
        return self.env

    def set(self, key, value):
        """
        Manually sets a value in the target environment.
        """
        self.env[key] = value

    def append(self, key, addition):
        """
        Manually adds a value to the environment string
        """
        if key in list(self.env.keys()):
            self.env[key] = os.pathsep.join([self.env[key], addition])
        else:
            self.set(key, addition)

#
# Miscellaneious utilities
#


def get_user(default=None):
    """
    Returns the username. For most purposes, this value
    will be the same as getpass.getuser on Unix and
    win32api.GetUserName on Windows, but in some situations
    (when running without a terminal, etc) getuser may throw
    a KeyError. In which case, or if the username resolves to
    False, the default value be returned.

    Any unexpected exceptions will be thrown.

    See ticket:6307
    """
    rv = None
    try:
        import getpass
        rv = getpass.getuser()  # Uses environment variable or pwd
    except KeyError:  # 6307, probably system
        pass
    except ImportError:  # No pwd on Windows
        import win32api
        rv = win32api.GetUserName()

    if not rv:
        return default
    else:
        return rv


def get_omero_userdir():
    """
    Returns the OMERO user directory

    In 6.0.0 the default will change to use appdirs.user_data_dir.
    You can enable this behaviour now by setting OMERO_USERDIR="" (empty
    string) instead of unset.

    Note that files in the old user directory will not be migrated.
    """
    omero_userdir = os.environ.get('OMERO_USERDIR', None)
    if omero_userdir:
        return path.path(omero_userdir)
    elif omero_userdir == "":
        return path.path(user_data_dir(*APPDIR_DEFAULTS))
    else:
        return old_div(path.path(get_user_dir()), "omero")


def get_omero_user_cache_dir():
    """Returns the OMERO user cache directory"""
    omero_userdir = os.environ.get('OMERO_USERDIR', None)
    if omero_userdir:
        return path.path(omero_userdir) / "cache"
    else:
        return path.path(user_cache_dir(*APPDIR_DEFAULTS))


# Other application directories may be added in future, see
# https://pypi.org/project/appdirs/


def get_user_dir():
    exceptions_to_handle = (ImportError)
    try:
        from pywintypes import com_error
        from win32com.shell import shellcon, shell
        exceptions_to_handle = (ImportError, com_error)
        homeprop = shell.SHGetFolderPath(0, shellcon.CSIDL_APPDATA, 0, 0)
    except exceptions_to_handle:
        homeprop = os.path.expanduser("~")

    if "~" == homeprop:
        # ticket:5583
        raise Exception("Unexpanded '~' from expanduser: see ticket:5583")

    return homeprop


def edit_path(path_or_obj, start_text):
    f = path.path(path_or_obj)
    editor = os.getenv("VISUAL") or os.getenv("EDITOR")
    if not editor:
        if platform.system() == "Windows":
            editor = "Notepad.exe"
        else:
            editor = "vi"

    if isinstance(start_text, str):
        start_text = start_text.encode("utf-8")
    f.write_text(start_text)

    # If absolute, then use the path
    # as is (ticket:4246). Otherwise,
    # use which.py to find it.
    editor_parts = shlex.split(editor)
    editor = editor_parts[0]
    editor_obj = path.path(editor)
    if editor_obj.isabs():
        editor_parts[0] = editor
    else:
        from omero_ext.which import which
        editor_parts[0] = which(editor)
    editor_parts.append(f)

    pid = os.spawnl(os.P_WAIT, editor_parts[0], *tuple(editor_parts))
    if pid:
        re = RuntimeError(
            "Couldn't spawn editor: %s" % editor_parts[0])
        re.pid = pid
        raise re

# From: http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/157035


def tail_lines(filename, linesback=10, returnlist=0):
    """Does what "tail -10 filename" would have done
       Parameters:

            filename   file to read
            linesback  Number of lines to read from end of file
            returnlist Return a list containing the lines instead of a string

    """
    avgcharsperline = 75

    file = open(filename, 'r')
    while 1:
        try:
            file.seek(-1 * avgcharsperline * linesback, 2)
        except IOError:
            file.seek(0)
        if file.tell() == 0:
            atstart = 1
        else:
            atstart = 0

        lines = file.read().split("\n")
        if (len(lines) > (linesback + 1)) or atstart:
            break
        # The lines are bigger than we thought
        avgcharsperline = avgcharsperline * 1.3  # Inc avg for retry
    file.close()

    if len(lines) > linesback:
        start = len(lines) - linesback - 1
    else:
        start = 0
    if returnlist:
        return lines[start:len(lines) - 1]

    out = ""
    for l in lines[start:len(lines) - 1]:
        out = out + l + "\n"
    return out



# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `Objective.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_model_IObject_ice
import omero_RTypes_ice
import omero_model_RTypes_ice
import omero_System_ice
import omero_Collections_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Start of module omero
__name__ = 'omero'

# Start of module omero.model
__name__ = 'omero.model'

if 'Immersion' not in _M_omero.model.__dict__:
    _M_omero.model._t_Immersion = IcePy.declareClass('::omero::model::Immersion')
    _M_omero.model._t_ImmersionPrx = IcePy.declareProxy('::omero::model::Immersion')

if 'Correction' not in _M_omero.model.__dict__:
    _M_omero.model._t_Correction = IcePy.declareClass('::omero::model::Correction')
    _M_omero.model._t_CorrectionPrx = IcePy.declareProxy('::omero::model::Correction')

if 'Length' not in _M_omero.model.__dict__:
    _M_omero.model._t_Length = IcePy.declareClass('::omero::model::Length')
    _M_omero.model._t_LengthPrx = IcePy.declareProxy('::omero::model::Length')

if 'Instrument' not in _M_omero.model.__dict__:
    _M_omero.model._t_Instrument = IcePy.declareClass('::omero::model::Instrument')
    _M_omero.model._t_InstrumentPrx = IcePy.declareProxy('::omero::model::Instrument')

if 'ObjectiveAnnotationLink' not in _M_omero.model.__dict__:
    _M_omero.model._t_ObjectiveAnnotationLink = IcePy.declareClass('::omero::model::ObjectiveAnnotationLink')
    _M_omero.model._t_ObjectiveAnnotationLinkPrx = IcePy.declareProxy('::omero::model::ObjectiveAnnotationLink')

if 'Annotation' not in _M_omero.model.__dict__:
    _M_omero.model._t_Annotation = IcePy.declareClass('::omero::model::Annotation')
    _M_omero.model._t_AnnotationPrx = IcePy.declareProxy('::omero::model::Annotation')

if 'Details' not in _M_omero.model.__dict__:
    _M_omero.model._t_Details = IcePy.declareClass('::omero::model::Details')
    _M_omero.model._t_DetailsPrx = IcePy.declareProxy('::omero::model::Details')

if '_t_ObjectiveAnnotationLinksSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_ObjectiveAnnotationLinksSeq = IcePy.defineSequence('::omero::model::ObjectiveAnnotationLinksSeq', (), _M_omero.model._t_ObjectiveAnnotationLink)

if '_t_ObjectiveLinkedAnnotationSeq' not in _M_omero.model.__dict__:
    _M_omero.model._t_ObjectiveLinkedAnnotationSeq = IcePy.defineSequence('::omero::model::ObjectiveLinkedAnnotationSeq', (), _M_omero.model._t_Annotation)

if 'Objective' not in _M_omero.model.__dict__:
    _M_omero.model.Objective = Ice.createTempClass()
    class Objective(_M_omero.model.IObject):
        def __init__(self, _id=None, _details=None, _loaded=False, _version=None, _manufacturer=None, _model=None, _lotNumber=None, _serialNumber=None, _nominalMagnification=None, _calibratedMagnification=None, _lensNA=None, _immersion=None, _correction=None, _workingDistance=None, _iris=None, _instrument=None, _annotationLinksSeq=None, _annotationLinksLoaded=False, _annotationLinksCountPerOwner=None):
            if Ice.getType(self) == _M_omero.model.Objective:
                raise RuntimeError('omero.model.Objective is an abstract class')
            _M_omero.model.IObject.__init__(self, _id, _details, _loaded)
            self._version = _version
            self._manufacturer = _manufacturer
            self._model = _model
            self._lotNumber = _lotNumber
            self._serialNumber = _serialNumber
            self._nominalMagnification = _nominalMagnification
            self._calibratedMagnification = _calibratedMagnification
            self._lensNA = _lensNA
            self._immersion = _immersion
            self._correction = _correction
            self._workingDistance = _workingDistance
            self._iris = _iris
            self._instrument = _instrument
            self._annotationLinksSeq = _annotationLinksSeq
            self._annotationLinksLoaded = _annotationLinksLoaded
            self._annotationLinksCountPerOwner = _annotationLinksCountPerOwner

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::model::IObject', '::omero::model::Objective')

        def ice_id(self, current=None):
            return '::omero::model::Objective'

        def ice_staticId():
            return '::omero::model::Objective'
        ice_staticId = staticmethod(ice_staticId)

        def getVersion(self, current=None):
            pass

        def setVersion(self, theVersion, current=None):
            pass

        def getManufacturer(self, current=None):
            pass

        def setManufacturer(self, theManufacturer, current=None):
            pass

        def getModel(self, current=None):
            pass

        def setModel(self, theModel, current=None):
            pass

        def getLotNumber(self, current=None):
            pass

        def setLotNumber(self, theLotNumber, current=None):
            pass

        def getSerialNumber(self, current=None):
            pass

        def setSerialNumber(self, theSerialNumber, current=None):
            pass

        def getNominalMagnification(self, current=None):
            pass

        def setNominalMagnification(self, theNominalMagnification, current=None):
            pass

        def getCalibratedMagnification(self, current=None):
            pass

        def setCalibratedMagnification(self, theCalibratedMagnification, current=None):
            pass

        def getLensNA(self, current=None):
            pass

        def setLensNA(self, theLensNA, current=None):
            pass

        def getImmersion(self, current=None):
            pass

        def setImmersion(self, theImmersion, current=None):
            pass

        def getCorrection(self, current=None):
            pass

        def setCorrection(self, theCorrection, current=None):
            pass

        def getWorkingDistance(self, current=None):
            pass

        def setWorkingDistance(self, theWorkingDistance, current=None):
            pass

        def getIris(self, current=None):
            pass

        def setIris(self, theIris, current=None):
            pass

        def getInstrument(self, current=None):
            pass

        def setInstrument(self, theInstrument, current=None):
            pass

        def unloadAnnotationLinks(self, current=None):
            pass

        def sizeOfAnnotationLinks(self, current=None):
            pass

        def copyAnnotationLinks(self, current=None):
            pass

        def addObjectiveAnnotationLink(self, target, current=None):
            pass

        def addAllObjectiveAnnotationLinkSet(self, targets, current=None):
            pass

        def removeObjectiveAnnotationLink(self, theTarget, current=None):
            pass

        def removeAllObjectiveAnnotationLinkSet(self, targets, current=None):
            pass

        def clearAnnotationLinks(self, current=None):
            pass

        def reloadAnnotationLinks(self, toCopy, current=None):
            pass

        def getAnnotationLinksCountPerOwner(self, current=None):
            pass

        def linkAnnotation(self, addition, current=None):
            pass

        def addObjectiveAnnotationLinkToBoth(self, link, bothSides, current=None):
            pass

        def findObjectiveAnnotationLink(self, removal, current=None):
            pass

        def unlinkAnnotation(self, removal, current=None):
            pass

        def removeObjectiveAnnotationLinkFromBoth(self, link, bothSides, current=None):
            pass

        def linkedAnnotationList(self, current=None):
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.model._t_Objective)

        __repr__ = __str__

    _M_omero.model.ObjectivePrx = Ice.createTempClass()
    class ObjectivePrx(_M_omero.model.IObjectPrx):

        def getVersion(self, _ctx=None):
            return _M_omero.model.Objective._op_getVersion.invoke(self, ((), _ctx))

        def begin_getVersion(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_getVersion.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getVersion(self, _r):
            return _M_omero.model.Objective._op_getVersion.end(self, _r)

        def setVersion(self, theVersion, _ctx=None):
            return _M_omero.model.Objective._op_setVersion.invoke(self, ((theVersion, ), _ctx))

        def begin_setVersion(self, theVersion, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_setVersion.begin(self, ((theVersion, ), _response, _ex, _sent, _ctx))

        def end_setVersion(self, _r):
            return _M_omero.model.Objective._op_setVersion.end(self, _r)

        def getManufacturer(self, _ctx=None):
            return _M_omero.model.Objective._op_getManufacturer.invoke(self, ((), _ctx))

        def begin_getManufacturer(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_getManufacturer.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getManufacturer(self, _r):
            return _M_omero.model.Objective._op_getManufacturer.end(self, _r)

        def setManufacturer(self, theManufacturer, _ctx=None):
            return _M_omero.model.Objective._op_setManufacturer.invoke(self, ((theManufacturer, ), _ctx))

        def begin_setManufacturer(self, theManufacturer, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_setManufacturer.begin(self, ((theManufacturer, ), _response, _ex, _sent, _ctx))

        def end_setManufacturer(self, _r):
            return _M_omero.model.Objective._op_setManufacturer.end(self, _r)

        def getModel(self, _ctx=None):
            return _M_omero.model.Objective._op_getModel.invoke(self, ((), _ctx))

        def begin_getModel(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_getModel.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getModel(self, _r):
            return _M_omero.model.Objective._op_getModel.end(self, _r)

        def setModel(self, theModel, _ctx=None):
            return _M_omero.model.Objective._op_setModel.invoke(self, ((theModel, ), _ctx))

        def begin_setModel(self, theModel, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_setModel.begin(self, ((theModel, ), _response, _ex, _sent, _ctx))

        def end_setModel(self, _r):
            return _M_omero.model.Objective._op_setModel.end(self, _r)

        def getLotNumber(self, _ctx=None):
            return _M_omero.model.Objective._op_getLotNumber.invoke(self, ((), _ctx))

        def begin_getLotNumber(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_getLotNumber.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getLotNumber(self, _r):
            return _M_omero.model.Objective._op_getLotNumber.end(self, _r)

        def setLotNumber(self, theLotNumber, _ctx=None):
            return _M_omero.model.Objective._op_setLotNumber.invoke(self, ((theLotNumber, ), _ctx))

        def begin_setLotNumber(self, theLotNumber, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_setLotNumber.begin(self, ((theLotNumber, ), _response, _ex, _sent, _ctx))

        def end_setLotNumber(self, _r):
            return _M_omero.model.Objective._op_setLotNumber.end(self, _r)

        def getSerialNumber(self, _ctx=None):
            return _M_omero.model.Objective._op_getSerialNumber.invoke(self, ((), _ctx))

        def begin_getSerialNumber(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_getSerialNumber.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getSerialNumber(self, _r):
            return _M_omero.model.Objective._op_getSerialNumber.end(self, _r)

        def setSerialNumber(self, theSerialNumber, _ctx=None):
            return _M_omero.model.Objective._op_setSerialNumber.invoke(self, ((theSerialNumber, ), _ctx))

        def begin_setSerialNumber(self, theSerialNumber, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_setSerialNumber.begin(self, ((theSerialNumber, ), _response, _ex, _sent, _ctx))

        def end_setSerialNumber(self, _r):
            return _M_omero.model.Objective._op_setSerialNumber.end(self, _r)

        def getNominalMagnification(self, _ctx=None):
            return _M_omero.model.Objective._op_getNominalMagnification.invoke(self, ((), _ctx))

        def begin_getNominalMagnification(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_getNominalMagnification.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getNominalMagnification(self, _r):
            return _M_omero.model.Objective._op_getNominalMagnification.end(self, _r)

        def setNominalMagnification(self, theNominalMagnification, _ctx=None):
            return _M_omero.model.Objective._op_setNominalMagnification.invoke(self, ((theNominalMagnification, ), _ctx))

        def begin_setNominalMagnification(self, theNominalMagnification, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_setNominalMagnification.begin(self, ((theNominalMagnification, ), _response, _ex, _sent, _ctx))

        def end_setNominalMagnification(self, _r):
            return _M_omero.model.Objective._op_setNominalMagnification.end(self, _r)

        def getCalibratedMagnification(self, _ctx=None):
            return _M_omero.model.Objective._op_getCalibratedMagnification.invoke(self, ((), _ctx))

        def begin_getCalibratedMagnification(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_getCalibratedMagnification.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getCalibratedMagnification(self, _r):
            return _M_omero.model.Objective._op_getCalibratedMagnification.end(self, _r)

        def setCalibratedMagnification(self, theCalibratedMagnification, _ctx=None):
            return _M_omero.model.Objective._op_setCalibratedMagnification.invoke(self, ((theCalibratedMagnification, ), _ctx))

        def begin_setCalibratedMagnification(self, theCalibratedMagnification, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_setCalibratedMagnification.begin(self, ((theCalibratedMagnification, ), _response, _ex, _sent, _ctx))

        def end_setCalibratedMagnification(self, _r):
            return _M_omero.model.Objective._op_setCalibratedMagnification.end(self, _r)

        def getLensNA(self, _ctx=None):
            return _M_omero.model.Objective._op_getLensNA.invoke(self, ((), _ctx))

        def begin_getLensNA(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_getLensNA.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getLensNA(self, _r):
            return _M_omero.model.Objective._op_getLensNA.end(self, _r)

        def setLensNA(self, theLensNA, _ctx=None):
            return _M_omero.model.Objective._op_setLensNA.invoke(self, ((theLensNA, ), _ctx))

        def begin_setLensNA(self, theLensNA, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_setLensNA.begin(self, ((theLensNA, ), _response, _ex, _sent, _ctx))

        def end_setLensNA(self, _r):
            return _M_omero.model.Objective._op_setLensNA.end(self, _r)

        def getImmersion(self, _ctx=None):
            return _M_omero.model.Objective._op_getImmersion.invoke(self, ((), _ctx))

        def begin_getImmersion(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_getImmersion.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getImmersion(self, _r):
            return _M_omero.model.Objective._op_getImmersion.end(self, _r)

        def setImmersion(self, theImmersion, _ctx=None):
            return _M_omero.model.Objective._op_setImmersion.invoke(self, ((theImmersion, ), _ctx))

        def begin_setImmersion(self, theImmersion, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_setImmersion.begin(self, ((theImmersion, ), _response, _ex, _sent, _ctx))

        def end_setImmersion(self, _r):
            return _M_omero.model.Objective._op_setImmersion.end(self, _r)

        def getCorrection(self, _ctx=None):
            return _M_omero.model.Objective._op_getCorrection.invoke(self, ((), _ctx))

        def begin_getCorrection(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_getCorrection.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getCorrection(self, _r):
            return _M_omero.model.Objective._op_getCorrection.end(self, _r)

        def setCorrection(self, theCorrection, _ctx=None):
            return _M_omero.model.Objective._op_setCorrection.invoke(self, ((theCorrection, ), _ctx))

        def begin_setCorrection(self, theCorrection, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_setCorrection.begin(self, ((theCorrection, ), _response, _ex, _sent, _ctx))

        def end_setCorrection(self, _r):
            return _M_omero.model.Objective._op_setCorrection.end(self, _r)

        def getWorkingDistance(self, _ctx=None):
            return _M_omero.model.Objective._op_getWorkingDistance.invoke(self, ((), _ctx))

        def begin_getWorkingDistance(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_getWorkingDistance.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getWorkingDistance(self, _r):
            return _M_omero.model.Objective._op_getWorkingDistance.end(self, _r)

        def setWorkingDistance(self, theWorkingDistance, _ctx=None):
            return _M_omero.model.Objective._op_setWorkingDistance.invoke(self, ((theWorkingDistance, ), _ctx))

        def begin_setWorkingDistance(self, theWorkingDistance, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_setWorkingDistance.begin(self, ((theWorkingDistance, ), _response, _ex, _sent, _ctx))

        def end_setWorkingDistance(self, _r):
            return _M_omero.model.Objective._op_setWorkingDistance.end(self, _r)

        def getIris(self, _ctx=None):
            return _M_omero.model.Objective._op_getIris.invoke(self, ((), _ctx))

        def begin_getIris(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_getIris.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getIris(self, _r):
            return _M_omero.model.Objective._op_getIris.end(self, _r)

        def setIris(self, theIris, _ctx=None):
            return _M_omero.model.Objective._op_setIris.invoke(self, ((theIris, ), _ctx))

        def begin_setIris(self, theIris, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_setIris.begin(self, ((theIris, ), _response, _ex, _sent, _ctx))

        def end_setIris(self, _r):
            return _M_omero.model.Objective._op_setIris.end(self, _r)

        def getInstrument(self, _ctx=None):
            return _M_omero.model.Objective._op_getInstrument.invoke(self, ((), _ctx))

        def begin_getInstrument(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_getInstrument.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getInstrument(self, _r):
            return _M_omero.model.Objective._op_getInstrument.end(self, _r)

        def setInstrument(self, theInstrument, _ctx=None):
            return _M_omero.model.Objective._op_setInstrument.invoke(self, ((theInstrument, ), _ctx))

        def begin_setInstrument(self, theInstrument, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_setInstrument.begin(self, ((theInstrument, ), _response, _ex, _sent, _ctx))

        def end_setInstrument(self, _r):
            return _M_omero.model.Objective._op_setInstrument.end(self, _r)

        def unloadAnnotationLinks(self, _ctx=None):
            return _M_omero.model.Objective._op_unloadAnnotationLinks.invoke(self, ((), _ctx))

        def begin_unloadAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_unloadAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_unloadAnnotationLinks(self, _r):
            return _M_omero.model.Objective._op_unloadAnnotationLinks.end(self, _r)

        def sizeOfAnnotationLinks(self, _ctx=None):
            return _M_omero.model.Objective._op_sizeOfAnnotationLinks.invoke(self, ((), _ctx))

        def begin_sizeOfAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_sizeOfAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_sizeOfAnnotationLinks(self, _r):
            return _M_omero.model.Objective._op_sizeOfAnnotationLinks.end(self, _r)

        def copyAnnotationLinks(self, _ctx=None):
            return _M_omero.model.Objective._op_copyAnnotationLinks.invoke(self, ((), _ctx))

        def begin_copyAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_copyAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_copyAnnotationLinks(self, _r):
            return _M_omero.model.Objective._op_copyAnnotationLinks.end(self, _r)

        def addObjectiveAnnotationLink(self, target, _ctx=None):
            return _M_omero.model.Objective._op_addObjectiveAnnotationLink.invoke(self, ((target, ), _ctx))

        def begin_addObjectiveAnnotationLink(self, target, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_addObjectiveAnnotationLink.begin(self, ((target, ), _response, _ex, _sent, _ctx))

        def end_addObjectiveAnnotationLink(self, _r):
            return _M_omero.model.Objective._op_addObjectiveAnnotationLink.end(self, _r)

        def addAllObjectiveAnnotationLinkSet(self, targets, _ctx=None):
            return _M_omero.model.Objective._op_addAllObjectiveAnnotationLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_addAllObjectiveAnnotationLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_addAllObjectiveAnnotationLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_addAllObjectiveAnnotationLinkSet(self, _r):
            return _M_omero.model.Objective._op_addAllObjectiveAnnotationLinkSet.end(self, _r)

        def removeObjectiveAnnotationLink(self, theTarget, _ctx=None):
            return _M_omero.model.Objective._op_removeObjectiveAnnotationLink.invoke(self, ((theTarget, ), _ctx))

        def begin_removeObjectiveAnnotationLink(self, theTarget, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_removeObjectiveAnnotationLink.begin(self, ((theTarget, ), _response, _ex, _sent, _ctx))

        def end_removeObjectiveAnnotationLink(self, _r):
            return _M_omero.model.Objective._op_removeObjectiveAnnotationLink.end(self, _r)

        def removeAllObjectiveAnnotationLinkSet(self, targets, _ctx=None):
            return _M_omero.model.Objective._op_removeAllObjectiveAnnotationLinkSet.invoke(self, ((targets, ), _ctx))

        def begin_removeAllObjectiveAnnotationLinkSet(self, targets, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_removeAllObjectiveAnnotationLinkSet.begin(self, ((targets, ), _response, _ex, _sent, _ctx))

        def end_removeAllObjectiveAnnotationLinkSet(self, _r):
            return _M_omero.model.Objective._op_removeAllObjectiveAnnotationLinkSet.end(self, _r)

        def clearAnnotationLinks(self, _ctx=None):
            return _M_omero.model.Objective._op_clearAnnotationLinks.invoke(self, ((), _ctx))

        def begin_clearAnnotationLinks(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_clearAnnotationLinks.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_clearAnnotationLinks(self, _r):
            return _M_omero.model.Objective._op_clearAnnotationLinks.end(self, _r)

        def reloadAnnotationLinks(self, toCopy, _ctx=None):
            return _M_omero.model.Objective._op_reloadAnnotationLinks.invoke(self, ((toCopy, ), _ctx))

        def begin_reloadAnnotationLinks(self, toCopy, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_reloadAnnotationLinks.begin(self, ((toCopy, ), _response, _ex, _sent, _ctx))

        def end_reloadAnnotationLinks(self, _r):
            return _M_omero.model.Objective._op_reloadAnnotationLinks.end(self, _r)

        def getAnnotationLinksCountPerOwner(self, _ctx=None):
            return _M_omero.model.Objective._op_getAnnotationLinksCountPerOwner.invoke(self, ((), _ctx))

        def begin_getAnnotationLinksCountPerOwner(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_getAnnotationLinksCountPerOwner.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_getAnnotationLinksCountPerOwner(self, _r):
            return _M_omero.model.Objective._op_getAnnotationLinksCountPerOwner.end(self, _r)

        def linkAnnotation(self, addition, _ctx=None):
            return _M_omero.model.Objective._op_linkAnnotation.invoke(self, ((addition, ), _ctx))

        def begin_linkAnnotation(self, addition, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_linkAnnotation.begin(self, ((addition, ), _response, _ex, _sent, _ctx))

        def end_linkAnnotation(self, _r):
            return _M_omero.model.Objective._op_linkAnnotation.end(self, _r)

        def addObjectiveAnnotationLinkToBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.Objective._op_addObjectiveAnnotationLinkToBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_addObjectiveAnnotationLinkToBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_addObjectiveAnnotationLinkToBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_addObjectiveAnnotationLinkToBoth(self, _r):
            return _M_omero.model.Objective._op_addObjectiveAnnotationLinkToBoth.end(self, _r)

        def findObjectiveAnnotationLink(self, removal, _ctx=None):
            return _M_omero.model.Objective._op_findObjectiveAnnotationLink.invoke(self, ((removal, ), _ctx))

        def begin_findObjectiveAnnotationLink(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_findObjectiveAnnotationLink.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_findObjectiveAnnotationLink(self, _r):
            return _M_omero.model.Objective._op_findObjectiveAnnotationLink.end(self, _r)

        def unlinkAnnotation(self, removal, _ctx=None):
            return _M_omero.model.Objective._op_unlinkAnnotation.invoke(self, ((removal, ), _ctx))

        def begin_unlinkAnnotation(self, removal, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_unlinkAnnotation.begin(self, ((removal, ), _response, _ex, _sent, _ctx))

        def end_unlinkAnnotation(self, _r):
            return _M_omero.model.Objective._op_unlinkAnnotation.end(self, _r)

        def removeObjectiveAnnotationLinkFromBoth(self, link, bothSides, _ctx=None):
            return _M_omero.model.Objective._op_removeObjectiveAnnotationLinkFromBoth.invoke(self, ((link, bothSides), _ctx))

        def begin_removeObjectiveAnnotationLinkFromBoth(self, link, bothSides, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_removeObjectiveAnnotationLinkFromBoth.begin(self, ((link, bothSides), _response, _ex, _sent, _ctx))

        def end_removeObjectiveAnnotationLinkFromBoth(self, _r):
            return _M_omero.model.Objective._op_removeObjectiveAnnotationLinkFromBoth.end(self, _r)

        def linkedAnnotationList(self, _ctx=None):
            return _M_omero.model.Objective._op_linkedAnnotationList.invoke(self, ((), _ctx))

        def begin_linkedAnnotationList(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.model.Objective._op_linkedAnnotationList.begin(self, ((), _response, _ex, _sent, _ctx))

        def end_linkedAnnotationList(self, _r):
            return _M_omero.model.Objective._op_linkedAnnotationList.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.model.ObjectivePrx.ice_checkedCast(proxy, '::omero::model::Objective', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.model.ObjectivePrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::model::Objective'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.model._t_ObjectivePrx = IcePy.defineProxy('::omero::model::Objective', ObjectivePrx)

    _M_omero.model._t_Objective = IcePy.declareClass('::omero::model::Objective')

    _M_omero.model._t_Objective = IcePy.defineClass('::omero::model::Objective', Objective, -1, (), True, False, _M_omero.model._t_IObject, (), (
        ('_version', (), _M_omero._t_RInt, False, 0),
        ('_manufacturer', (), _M_omero._t_RString, False, 0),
        ('_model', (), _M_omero._t_RString, False, 0),
        ('_lotNumber', (), _M_omero._t_RString, False, 0),
        ('_serialNumber', (), _M_omero._t_RString, False, 0),
        ('_nominalMagnification', (), _M_omero._t_RDouble, False, 0),
        ('_calibratedMagnification', (), _M_omero._t_RDouble, False, 0),
        ('_lensNA', (), _M_omero._t_RDouble, False, 0),
        ('_immersion', (), _M_omero.model._t_Immersion, False, 0),
        ('_correction', (), _M_omero.model._t_Correction, False, 0),
        ('_workingDistance', (), _M_omero.model._t_Length, False, 0),
        ('_iris', (), _M_omero._t_RBool, False, 0),
        ('_instrument', (), _M_omero.model._t_Instrument, False, 0),
        ('_annotationLinksSeq', (), _M_omero.model._t_ObjectiveAnnotationLinksSeq, False, 0),
        ('_annotationLinksLoaded', (), IcePy._t_bool, False, 0),
        ('_annotationLinksCountPerOwner', (), _M_omero.sys._t_CountMap, False, 0)
    ))
    Objective._ice_type = _M_omero.model._t_Objective

    Objective._op_getVersion = IcePy.Operation('getVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RInt, False, 0), ())
    Objective._op_setVersion = IcePy.Operation('setVersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RInt, False, 0),), (), None, ())
    Objective._op_getManufacturer = IcePy.Operation('getManufacturer', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    Objective._op_setManufacturer = IcePy.Operation('setManufacturer', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    Objective._op_getModel = IcePy.Operation('getModel', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    Objective._op_setModel = IcePy.Operation('setModel', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    Objective._op_getLotNumber = IcePy.Operation('getLotNumber', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    Objective._op_setLotNumber = IcePy.Operation('setLotNumber', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    Objective._op_getSerialNumber = IcePy.Operation('getSerialNumber', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RString, False, 0), ())
    Objective._op_setSerialNumber = IcePy.Operation('setSerialNumber', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RString, False, 0),), (), None, ())
    Objective._op_getNominalMagnification = IcePy.Operation('getNominalMagnification', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RDouble, False, 0), ())
    Objective._op_setNominalMagnification = IcePy.Operation('setNominalMagnification', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RDouble, False, 0),), (), None, ())
    Objective._op_getCalibratedMagnification = IcePy.Operation('getCalibratedMagnification', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RDouble, False, 0), ())
    Objective._op_setCalibratedMagnification = IcePy.Operation('setCalibratedMagnification', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RDouble, False, 0),), (), None, ())
    Objective._op_getLensNA = IcePy.Operation('getLensNA', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RDouble, False, 0), ())
    Objective._op_setLensNA = IcePy.Operation('setLensNA', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RDouble, False, 0),), (), None, ())
    Objective._op_getImmersion = IcePy.Operation('getImmersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_Immersion, False, 0), ())
    Objective._op_setImmersion = IcePy.Operation('setImmersion', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Immersion, False, 0),), (), None, ())
    Objective._op_getCorrection = IcePy.Operation('getCorrection', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_Correction, False, 0), ())
    Objective._op_setCorrection = IcePy.Operation('setCorrection', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Correction, False, 0),), (), None, ())
    Objective._op_getWorkingDistance = IcePy.Operation('getWorkingDistance', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_Length, False, 0), ())
    Objective._op_setWorkingDistance = IcePy.Operation('setWorkingDistance', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Length, False, 0),), (), None, ())
    Objective._op_getIris = IcePy.Operation('getIris', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero._t_RBool, False, 0), ())
    Objective._op_setIris = IcePy.Operation('setIris', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero._t_RBool, False, 0),), (), None, ())
    Objective._op_getInstrument = IcePy.Operation('getInstrument', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_Instrument, False, 0), ())
    Objective._op_setInstrument = IcePy.Operation('setInstrument', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Instrument, False, 0),), (), None, ())
    Objective._op_unloadAnnotationLinks = IcePy.Operation('unloadAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Objective._op_sizeOfAnnotationLinks = IcePy.Operation('sizeOfAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), IcePy._t_int, False, 0), ())
    Objective._op_copyAnnotationLinks = IcePy.Operation('copyAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_ObjectiveAnnotationLinksSeq, False, 0), ())
    Objective._op_addObjectiveAnnotationLink = IcePy.Operation('addObjectiveAnnotationLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_ObjectiveAnnotationLink, False, 0),), (), None, ())
    Objective._op_addAllObjectiveAnnotationLinkSet = IcePy.Operation('addAllObjectiveAnnotationLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_ObjectiveAnnotationLinksSeq, False, 0),), (), None, ())
    Objective._op_removeObjectiveAnnotationLink = IcePy.Operation('removeObjectiveAnnotationLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_ObjectiveAnnotationLink, False, 0),), (), None, ())
    Objective._op_removeAllObjectiveAnnotationLinkSet = IcePy.Operation('removeAllObjectiveAnnotationLinkSet', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_ObjectiveAnnotationLinksSeq, False, 0),), (), None, ())
    Objective._op_clearAnnotationLinks = IcePy.Operation('clearAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), None, ())
    Objective._op_reloadAnnotationLinks = IcePy.Operation('reloadAnnotationLinks', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Objective, False, 0),), (), None, ())
    Objective._op_getAnnotationLinksCountPerOwner = IcePy.Operation('getAnnotationLinksCountPerOwner', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.sys._t_CountMap, False, 0), ())
    Objective._op_linkAnnotation = IcePy.Operation('linkAnnotation', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Annotation, False, 0),), (), ((), _M_omero.model._t_ObjectiveAnnotationLink, False, 0), ())
    Objective._op_addObjectiveAnnotationLinkToBoth = IcePy.Operation('addObjectiveAnnotationLinkToBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_ObjectiveAnnotationLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    Objective._op_findObjectiveAnnotationLink = IcePy.Operation('findObjectiveAnnotationLink', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Annotation, False, 0),), (), ((), _M_omero.model._t_ObjectiveAnnotationLinksSeq, False, 0), ())
    Objective._op_unlinkAnnotation = IcePy.Operation('unlinkAnnotation', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Annotation, False, 0),), (), None, ())
    Objective._op_removeObjectiveAnnotationLinkFromBoth = IcePy.Operation('removeObjectiveAnnotationLinkFromBoth', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_ObjectiveAnnotationLink, False, 0), ((), IcePy._t_bool, False, 0)), (), None, ())
    Objective._op_linkedAnnotationList = IcePy.Operation('linkedAnnotationList', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (), (), ((), _M_omero.model._t_ObjectiveLinkedAnnotationSeq, False, 0), ())

    _M_omero.model.Objective = Objective
    del Objective

    _M_omero.model.ObjectivePrx = ObjectivePrx
    del ObjectivePrx

# End of module omero.model

__name__ = 'omero'

# End of module omero



# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `SharedResources.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_Repositories_ice
import omero_Scripts_ice
import omero_Tables_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module Glacier2
_M_Glacier2 = Ice.openModule('Glacier2')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Included module omero.grid
_M_omero.grid = Ice.openModule('omero.grid')

# Included module omero.cmd
_M_omero.cmd = Ice.openModule('omero.cmd')

# Start of module omero
__name__ = 'omero'

# Start of module omero.grid
__name__ = 'omero.grid'

if 'SharedResources' not in _M_omero.grid.__dict__:
    _M_omero.grid.SharedResources = Ice.createTempClass()
    class SharedResources(Ice.Object):
        """
        Resource manager provided by each Blitz session for acquiring
        shared resources in the OmeroGrid. Unlike the other services
        provided by ServiceFactory instances, it is not guaranteed
        that a service instance returned from this interface will be
        returned if that resource happens to be busy. In that case,
        a null will be returned.
        """
        def __init__(self):
            if Ice.getType(self) == _M_omero.grid.SharedResources:
                raise RuntimeError('omero.grid.SharedResources is an abstract class')

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::grid::SharedResources')

        def ice_id(self, current=None):
            return '::omero::grid::SharedResources'

        def ice_staticId():
            return '::omero::grid::SharedResources'
        ice_staticId = staticmethod(ice_staticId)

        def acquireProcessor(self, job, seconds, current=None):
            """
            Waits up to seconds to acquire a slot in a processor
            which can handle the given job.
            Arguments:
            job -- 
            seconds -- 
            current -- The Current object for the invocation.
            """
            pass

        def addProcessor(self, proc, current=None):
            """
            Registers a omero.grid.Processor for Storm notifications
            so that other sessions can query whether or not a given
            processor would accept a given task.
            Arguments:
            proc -- 
            current -- The Current object for the invocation.
            """
            pass

        def removeProcessor(self, proc, current=None):
            """
            Unregisters a omero.grid.Processor from Storm
            notifications. If the processor was not already registered via
            {@code addProcessor} this is a no-op.
            Arguments:
            proc -- 
            current -- The Current object for the invocation.
            """
            pass

        def repositories(self, current=None):
            """
            Returns a map between Repository descriptions (omero::model::OriginalFile
            instances) and RepositoryPrx instances (possibly null).
            Arguments:
            current -- The Current object for the invocation.
            """
            pass

        def getScriptRepository(self, current=None):
            """
            Returns the single (possibly mirrored) script repository which makes
            all official scripts available.
            Arguments:
            current -- The Current object for the invocation.
            """
            pass

        def areTablesEnabled(self, current=None):
            """
            Returns true if a {@code Tables} service is active in the grid.
            If this value is false, then all calls to {@code #ewTable}
            or {@code openTable} will either fail or return null (possibly
            blocking while waiting for a service to startup)
            Arguments:
            current -- The Current object for the invocation.
            """
            pass

        def newTable(self, repoId, path, current=None):
            """
            Creates a new Format(""OMERO.tables"") file at the given path
            on the given repository. The returned Table proxy follows
            the same semantics as the openTable method.
            Arguments:
            repoId -- 
            path -- 
            current -- The Current object for the invocation.
            """
            pass

        def openTable(self, file, current=None):
            """
            Returns a Table instance or null. Table instances are not
            exclusively owned by the client and may throw an OptimisticLockException
            if background modifications take place.
            The file instance must be managed (i.e. have a non-null id) and
            be of the format ""OMERO.tables"". Use newTable() to create
            a new instance.
            Arguments:
            file -- 
            current -- The Current object for the invocation.
            """
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.grid._t_SharedResources)

        __repr__ = __str__

    _M_omero.grid.SharedResourcesPrx = Ice.createTempClass()
    class SharedResourcesPrx(Ice.ObjectPrx):

        """
        Waits up to seconds to acquire a slot in a processor
        which can handle the given job.
        Arguments:
        job -- 
        seconds -- 
        _ctx -- The request context for the invocation.
        """
        def acquireProcessor(self, job, seconds, _ctx=None):
            return _M_omero.grid.SharedResources._op_acquireProcessor.invoke(self, ((job, seconds), _ctx))

        """
        Waits up to seconds to acquire a slot in a processor
        which can handle the given job.
        Arguments:
        job -- 
        seconds -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_acquireProcessor(self, job, seconds, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.SharedResources._op_acquireProcessor.begin(self, ((job, seconds), _response, _ex, _sent, _ctx))

        """
        Waits up to seconds to acquire a slot in a processor
        which can handle the given job.
        Arguments:
        job -- 
        seconds -- 
        """
        def end_acquireProcessor(self, _r):
            return _M_omero.grid.SharedResources._op_acquireProcessor.end(self, _r)

        """
        Registers a omero.grid.Processor for Storm notifications
        so that other sessions can query whether or not a given
        processor would accept a given task.
        Arguments:
        proc -- 
        _ctx -- The request context for the invocation.
        """
        def addProcessor(self, proc, _ctx=None):
            return _M_omero.grid.SharedResources._op_addProcessor.invoke(self, ((proc, ), _ctx))

        """
        Registers a omero.grid.Processor for Storm notifications
        so that other sessions can query whether or not a given
        processor would accept a given task.
        Arguments:
        proc -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_addProcessor(self, proc, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.SharedResources._op_addProcessor.begin(self, ((proc, ), _response, _ex, _sent, _ctx))

        """
        Registers a omero.grid.Processor for Storm notifications
        so that other sessions can query whether or not a given
        processor would accept a given task.
        Arguments:
        proc -- 
        """
        def end_addProcessor(self, _r):
            return _M_omero.grid.SharedResources._op_addProcessor.end(self, _r)

        """
        Unregisters a omero.grid.Processor from Storm
        notifications. If the processor was not already registered via
        {@code addProcessor} this is a no-op.
        Arguments:
        proc -- 
        _ctx -- The request context for the invocation.
        """
        def removeProcessor(self, proc, _ctx=None):
            return _M_omero.grid.SharedResources._op_removeProcessor.invoke(self, ((proc, ), _ctx))

        """
        Unregisters a omero.grid.Processor from Storm
        notifications. If the processor was not already registered via
        {@code addProcessor} this is a no-op.
        Arguments:
        proc -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_removeProcessor(self, proc, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.SharedResources._op_removeProcessor.begin(self, ((proc, ), _response, _ex, _sent, _ctx))

        """
        Unregisters a omero.grid.Processor from Storm
        notifications. If the processor was not already registered via
        {@code addProcessor} this is a no-op.
        Arguments:
        proc -- 
        """
        def end_removeProcessor(self, _r):
            return _M_omero.grid.SharedResources._op_removeProcessor.end(self, _r)

        """
        Returns a map between Repository descriptions (omero::model::OriginalFile
        instances) and RepositoryPrx instances (possibly null).
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def repositories(self, _ctx=None):
            return _M_omero.grid.SharedResources._op_repositories.invoke(self, ((), _ctx))

        """
        Returns a map between Repository descriptions (omero::model::OriginalFile
        instances) and RepositoryPrx instances (possibly null).
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_repositories(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.SharedResources._op_repositories.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Returns a map between Repository descriptions (omero::model::OriginalFile
        instances) and RepositoryPrx instances (possibly null).
        Arguments:
        """
        def end_repositories(self, _r):
            return _M_omero.grid.SharedResources._op_repositories.end(self, _r)

        """
        Returns the single (possibly mirrored) script repository which makes
        all official scripts available.
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def getScriptRepository(self, _ctx=None):
            return _M_omero.grid.SharedResources._op_getScriptRepository.invoke(self, ((), _ctx))

        """
        Returns the single (possibly mirrored) script repository which makes
        all official scripts available.
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_getScriptRepository(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.SharedResources._op_getScriptRepository.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Returns the single (possibly mirrored) script repository which makes
        all official scripts available.
        Arguments:
        """
        def end_getScriptRepository(self, _r):
            return _M_omero.grid.SharedResources._op_getScriptRepository.end(self, _r)

        """
        Returns true if a {@code Tables} service is active in the grid.
        If this value is false, then all calls to {@code #ewTable}
        or {@code openTable} will either fail or return null (possibly
        blocking while waiting for a service to startup)
        Arguments:
        _ctx -- The request context for the invocation.
        """
        def areTablesEnabled(self, _ctx=None):
            return _M_omero.grid.SharedResources._op_areTablesEnabled.invoke(self, ((), _ctx))

        """
        Returns true if a {@code Tables} service is active in the grid.
        If this value is false, then all calls to {@code #ewTable}
        or {@code openTable} will either fail or return null (possibly
        blocking while waiting for a service to startup)
        Arguments:
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_areTablesEnabled(self, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.SharedResources._op_areTablesEnabled.begin(self, ((), _response, _ex, _sent, _ctx))

        """
        Returns true if a {@code Tables} service is active in the grid.
        If this value is false, then all calls to {@code #ewTable}
        or {@code openTable} will either fail or return null (possibly
        blocking while waiting for a service to startup)
        Arguments:
        """
        def end_areTablesEnabled(self, _r):
            return _M_omero.grid.SharedResources._op_areTablesEnabled.end(self, _r)

        """
        Creates a new Format(""OMERO.tables"") file at the given path
        on the given repository. The returned Table proxy follows
        the same semantics as the openTable method.
        Arguments:
        repoId -- 
        path -- 
        _ctx -- The request context for the invocation.
        """
        def newTable(self, repoId, path, _ctx=None):
            return _M_omero.grid.SharedResources._op_newTable.invoke(self, ((repoId, path), _ctx))

        """
        Creates a new Format(""OMERO.tables"") file at the given path
        on the given repository. The returned Table proxy follows
        the same semantics as the openTable method.
        Arguments:
        repoId -- 
        path -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_newTable(self, repoId, path, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.SharedResources._op_newTable.begin(self, ((repoId, path), _response, _ex, _sent, _ctx))

        """
        Creates a new Format(""OMERO.tables"") file at the given path
        on the given repository. The returned Table proxy follows
        the same semantics as the openTable method.
        Arguments:
        repoId -- 
        path -- 
        """
        def end_newTable(self, _r):
            return _M_omero.grid.SharedResources._op_newTable.end(self, _r)

        """
        Returns a Table instance or null. Table instances are not
        exclusively owned by the client and may throw an OptimisticLockException
        if background modifications take place.
        The file instance must be managed (i.e. have a non-null id) and
        be of the format ""OMERO.tables"". Use newTable() to create
        a new instance.
        Arguments:
        file -- 
        _ctx -- The request context for the invocation.
        """
        def openTable(self, file, _ctx=None):
            return _M_omero.grid.SharedResources._op_openTable.invoke(self, ((file, ), _ctx))

        """
        Returns a Table instance or null. Table instances are not
        exclusively owned by the client and may throw an OptimisticLockException
        if background modifications take place.
        The file instance must be managed (i.e. have a non-null id) and
        be of the format ""OMERO.tables"". Use newTable() to create
        a new instance.
        Arguments:
        file -- 
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_openTable(self, file, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.grid.SharedResources._op_openTable.begin(self, ((file, ), _response, _ex, _sent, _ctx))

        """
        Returns a Table instance or null. Table instances are not
        exclusively owned by the client and may throw an OptimisticLockException
        if background modifications take place.
        The file instance must be managed (i.e. have a non-null id) and
        be of the format ""OMERO.tables"". Use newTable() to create
        a new instance.
        Arguments:
        file -- 
        """
        def end_openTable(self, _r):
            return _M_omero.grid.SharedResources._op_openTable.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.grid.SharedResourcesPrx.ice_checkedCast(proxy, '::omero::grid::SharedResources', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.grid.SharedResourcesPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::grid::SharedResources'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.grid._t_SharedResourcesPrx = IcePy.defineProxy('::omero::grid::SharedResources', SharedResourcesPrx)

    _M_omero.grid._t_SharedResources = IcePy.defineClass('::omero::grid::SharedResources', SharedResources, -1, (), True, False, None, (), ())
    SharedResources._ice_type = _M_omero.grid._t_SharedResources

    SharedResources._op_acquireProcessor = IcePy.Operation('acquireProcessor', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.model._t_Job, False, 0), ((), IcePy._t_int, False, 0)), (), ((), _M_omero.grid._t_InteractiveProcessorPrx, False, 0), (_M_omero._t_ServerError,))
    SharedResources._op_addProcessor = IcePy.Operation('addProcessor', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.grid._t_ProcessorPrx, False, 0),), (), None, (_M_omero._t_ServerError,))
    SharedResources._op_removeProcessor = IcePy.Operation('removeProcessor', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), _M_omero.grid._t_ProcessorPrx, False, 0),), (), None, (_M_omero._t_ServerError,))
    SharedResources._op_repositories = IcePy.Operation('repositories', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, False, None, (), (), (), ((), _M_omero.grid._t_RepositoryMap, False, 0), (_M_omero._t_ServerError,))
    SharedResources._op_getScriptRepository = IcePy.Operation('getScriptRepository', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, False, None, (), (), (), ((), _M_omero.grid._t_RepositoryPrx, False, 0), (_M_omero._t_ServerError,))
    SharedResources._op_areTablesEnabled = IcePy.Operation('areTablesEnabled', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, False, None, (), (), (), ((), IcePy._t_bool, False, 0), (_M_omero._t_ServerError,))
    SharedResources._op_newTable = IcePy.Operation('newTable', Ice.OperationMode.Normal, Ice.OperationMode.Normal, False, None, (), (((), IcePy._t_long, False, 0), ((), IcePy._t_string, False, 0)), (), ((), _M_omero.grid._t_TablePrx, False, 0), (_M_omero._t_ServerError,))
    SharedResources._op_openTable = IcePy.Operation('openTable', Ice.OperationMode.Idempotent, Ice.OperationMode.Idempotent, False, None, (), (((), _M_omero.model._t_OriginalFile, False, 0),), (), ((), _M_omero.grid._t_TablePrx, False, 0), (_M_omero._t_ServerError,))

    _M_omero.grid.SharedResources = SharedResources
    del SharedResources

    _M_omero.grid.SharedResourcesPrx = SharedResourcesPrx
    del SharedResourcesPrx

# End of module omero.grid

__name__ = 'omero'

# End of module omero



# -*- coding: utf-8 -*-
# **********************************************************************
#
# Copyright (c) 2003-2017 ZeroC, Inc. All rights reserved.
#
# This copy of Ice is licensed to you under the terms described in the
# ICE_LICENSE file included in this distribution.
#
# **********************************************************************
#
# Ice version 3.6.4
#
# <auto-generated>
#
# Generated from file `IProjection.ice'
#
# Warning: do not edit this file.
#
# </auto-generated>
#

from sys import version_info as _version_info_
import Ice, IcePy
import omero_ModelF_ice
import omero_ServicesF_ice
import omero_Collections_ice
import omero_Constants_ice

# Included module omero
_M_omero = Ice.openModule('omero')

# Included module omero.model
_M_omero.model = Ice.openModule('omero.model')

# Included module Ice
_M_Ice = Ice.openModule('Ice')

# Included module Glacier2
_M_Glacier2 = Ice.openModule('Glacier2')

# Included module omero.sys
_M_omero.sys = Ice.openModule('omero.sys')

# Included module omero.api
_M_omero.api = Ice.openModule('omero.api')

# Included module omero.grid
_M_omero.grid = Ice.openModule('omero.grid')

# Included module omero.constants
_M_omero.constants = Ice.openModule('omero.constants')

# Included module omero.constants.cluster
_M_omero.constants.cluster = Ice.openModule('omero.constants.cluster')

# Included module omero.constants.annotation
_M_omero.constants.annotation = Ice.openModule('omero.constants.annotation')

# Included module omero.constants.annotation.file
_M_omero.constants.annotation.file = Ice.openModule('omero.constants.annotation.file')

# Included module omero.constants.data
_M_omero.constants.data = Ice.openModule('omero.constants.data')

# Included module omero.constants.metadata
_M_omero.constants.metadata = Ice.openModule('omero.constants.metadata')

# Included module omero.constants.namespaces
_M_omero.constants.namespaces = Ice.openModule('omero.constants.namespaces')

# Included module omero.constants.analysis
_M_omero.constants.analysis = Ice.openModule('omero.constants.analysis')

# Included module omero.constants.analysis.flim
_M_omero.constants.analysis.flim = Ice.openModule('omero.constants.analysis.flim')

# Included module omero.constants.jobs
_M_omero.constants.jobs = Ice.openModule('omero.constants.jobs')

# Included module omero.constants.permissions
_M_omero.constants.permissions = Ice.openModule('omero.constants.permissions')

# Included module omero.constants.projection
_M_omero.constants.projection = Ice.openModule('omero.constants.projection')

# Included module omero.constants.topics
_M_omero.constants.topics = Ice.openModule('omero.constants.topics')

# Included module omero.constants.categories
_M_omero.constants.categories = Ice.openModule('omero.constants.categories')

# Start of module omero
__name__ = 'omero'

# Start of module omero.api
__name__ = 'omero.api'

if 'IProjection' not in _M_omero.api.__dict__:
    _M_omero.api.IProjection = Ice.createTempClass()
    class IProjection(_M_omero.api.ServiceInterface):
        """
        Provides methods for performing projections of Pixels sets.
        """
        def __init__(self):
            if Ice.getType(self) == _M_omero.api.IProjection:
                raise RuntimeError('omero.api.IProjection is an abstract class')

        def ice_ids(self, current=None):
            return ('::Ice::Object', '::omero::api::IProjection', '::omero::api::ServiceInterface')

        def ice_id(self, current=None):
            return '::omero::api::IProjection'

        def ice_staticId():
            return '::omero::api::IProjection'
        ice_staticId = staticmethod(ice_staticId)

        def projectStack_async(self, _cb, pixelsId, pixelsType, algorithm, timepoint, channelIndex, stepping, start, end, current=None):
            """
            Performs a projection through the optical sections of a
            particular wavelength at a given time point of a Pixels set.
            Arguments:
            _cb -- The asynchronous callback object.
            pixelsId -- The source Pixels set Id.
            pixelsType -- The destination Pixels type. If null, the source Pixels set pixels type will be used.
            algorithm -- MAXIMUM_INTENSITY, MEAN_INTENSITY or SUM_INTENSITY. NOTE: When performing a SUM_INTENSITY projection, pixel values will be pinned to the maximum pixel value of the destination Pixels type.
            timepoint -- Timepoint to perform the projection.
            channelIndex -- Index of the channel to perform the projection.
            stepping -- Stepping value to use while calculating the projection. For example, stepping=1 will use every optical section from start to end where stepping=2 will use every other section from start to end to perform the projection.
            start -- Optical section to start projecting from.
            end -- Optical section to finish projecting.
            current -- The Current object for the invocation.
            Throws:
            ValidationException -- Where: algorithm is unknown timepoint is out of range channelIndex is out of range start is out of range end is out of range start is greater than end the Pixels set qualified by pixelsId is not locatable.
            """
            pass

        def projectPixels_async(self, _cb, pixelsId, pixelsType, algorithm, tStart, tEnd, channelList, stepping, zStart, zEnd, name, current=None):
            """
            Performs a projection through selected optical sections and
            optical sections for a given set of time points of a Pixels
            set. The Image which is linked to the Pixels set will be
            copied using
            {@code omero.api.IPixels.copyAndResizeImage}.
            Arguments:
            _cb -- The asynchronous callback object.
            pixelsId -- The source Pixels set Id.
            pixelsType -- The destination Pixels type. If null, the source Pixels set pixels type will be used.
            algorithm -- MAXIMUM_INTENSITY, MEAN_INTENSITY or SUM_INTENSITY. NOTE: When performing a SUM_INTENSITY projection, pixel values will be pinned to the maximum pixel value of the destination Pixels type.
            tStart -- Timepoint to start projecting from.
            tEnd -- Timepoint to finish projecting.
            channelList -- List of the channel indexes to use while calculating the projection.
            stepping -- Stepping value to use while calculating the projection. For example, stepping=1 will use every optical section from start to end where stepping=2 will use every other section from start to end to perform the projection.
            zStart -- Optical section to start projecting from.
            zEnd -- Optical section to finish projecting.
            name -- Name for the newly created image. If null the name of the Image linked to the Pixels qualified by pixelsId will be used with a Projection suffix. For example, GFP-H2B Image of HeLa Cells will have an Image name of GFP-H2B Image of HeLa Cells Projection used for the projection.
            current -- The Current object for the invocation.
            Throws:
            ValidationException -- Where: algorithm is unknown tStart is out of range tEnd is out of range tStart is greater than tEnd channelList is null or has indexes out of range zStart is out of range zEnd is out of range zStart is greater than zEnd the Pixels set qualified by pixelsId is not locatable.
            """
            pass

        def __str__(self):
            return IcePy.stringify(self, _M_omero.api._t_IProjection)

        __repr__ = __str__

    _M_omero.api.IProjectionPrx = Ice.createTempClass()
    class IProjectionPrx(_M_omero.api.ServiceInterfacePrx):

        """
        Performs a projection through the optical sections of a
        particular wavelength at a given time point of a Pixels set.
        Arguments:
        pixelsId -- The source Pixels set Id.
        pixelsType -- The destination Pixels type. If null, the source Pixels set pixels type will be used.
        algorithm -- MAXIMUM_INTENSITY, MEAN_INTENSITY or SUM_INTENSITY. NOTE: When performing a SUM_INTENSITY projection, pixel values will be pinned to the maximum pixel value of the destination Pixels type.
        timepoint -- Timepoint to perform the projection.
        channelIndex -- Index of the channel to perform the projection.
        stepping -- Stepping value to use while calculating the projection. For example, stepping=1 will use every optical section from start to end where stepping=2 will use every other section from start to end to perform the projection.
        start -- Optical section to start projecting from.
        end -- Optical section to finish projecting.
        _ctx -- The request context for the invocation.
        Returns: A byte array of projected pixel values whose length is equal to the Pixels set 8         sizeX * sizeY * bytesPerPixel in big-endian format.
        Throws:
        ValidationException -- Where: algorithm is unknown timepoint is out of range channelIndex is out of range start is out of range end is out of range start is greater than end the Pixels set qualified by pixelsId is not locatable.
        """
        def projectStack(self, pixelsId, pixelsType, algorithm, timepoint, channelIndex, stepping, start, end, _ctx=None):
            return _M_omero.api.IProjection._op_projectStack.invoke(self, ((pixelsId, pixelsType, algorithm, timepoint, channelIndex, stepping, start, end), _ctx))

        """
        Performs a projection through the optical sections of a
        particular wavelength at a given time point of a Pixels set.
        Arguments:
        pixelsId -- The source Pixels set Id.
        pixelsType -- The destination Pixels type. If null, the source Pixels set pixels type will be used.
        algorithm -- MAXIMUM_INTENSITY, MEAN_INTENSITY or SUM_INTENSITY. NOTE: When performing a SUM_INTENSITY projection, pixel values will be pinned to the maximum pixel value of the destination Pixels type.
        timepoint -- Timepoint to perform the projection.
        channelIndex -- Index of the channel to perform the projection.
        stepping -- Stepping value to use while calculating the projection. For example, stepping=1 will use every optical section from start to end where stepping=2 will use every other section from start to end to perform the projection.
        start -- Optical section to start projecting from.
        end -- Optical section to finish projecting.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_projectStack(self, pixelsId, pixelsType, algorithm, timepoint, channelIndex, stepping, start, end, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IProjection._op_projectStack.begin(self, ((pixelsId, pixelsType, algorithm, timepoint, channelIndex, stepping, start, end), _response, _ex, _sent, _ctx))

        """
        Performs a projection through the optical sections of a
        particular wavelength at a given time point of a Pixels set.
        Arguments:
        pixelsId -- The source Pixels set Id.
        pixelsType -- The destination Pixels type. If null, the source Pixels set pixels type will be used.
        algorithm -- MAXIMUM_INTENSITY, MEAN_INTENSITY or SUM_INTENSITY. NOTE: When performing a SUM_INTENSITY projection, pixel values will be pinned to the maximum pixel value of the destination Pixels type.
        timepoint -- Timepoint to perform the projection.
        channelIndex -- Index of the channel to perform the projection.
        stepping -- Stepping value to use while calculating the projection. For example, stepping=1 will use every optical section from start to end where stepping=2 will use every other section from start to end to perform the projection.
        start -- Optical section to start projecting from.
        end -- Optical section to finish projecting.
        Returns: A byte array of projected pixel values whose length is equal to the Pixels set 8         sizeX * sizeY * bytesPerPixel in big-endian format.
        Throws:
        ValidationException -- Where: algorithm is unknown timepoint is out of range channelIndex is out of range start is out of range end is out of range start is greater than end the Pixels set qualified by pixelsId is not locatable.
        """
        def end_projectStack(self, _r):
            return _M_omero.api.IProjection._op_projectStack.end(self, _r)

        """
        Performs a projection through selected optical sections and
        optical sections for a given set of time points of a Pixels
        set. The Image which is linked to the Pixels set will be
        copied using
        {@code omero.api.IPixels.copyAndResizeImage}.
        Arguments:
        pixelsId -- The source Pixels set Id.
        pixelsType -- The destination Pixels type. If null, the source Pixels set pixels type will be used.
        algorithm -- MAXIMUM_INTENSITY, MEAN_INTENSITY or SUM_INTENSITY. NOTE: When performing a SUM_INTENSITY projection, pixel values will be pinned to the maximum pixel value of the destination Pixels type.
        tStart -- Timepoint to start projecting from.
        tEnd -- Timepoint to finish projecting.
        channelList -- List of the channel indexes to use while calculating the projection.
        stepping -- Stepping value to use while calculating the projection. For example, stepping=1 will use every optical section from start to end where stepping=2 will use every other section from start to end to perform the projection.
        zStart -- Optical section to start projecting from.
        zEnd -- Optical section to finish projecting.
        name -- Name for the newly created image. If null the name of the Image linked to the Pixels qualified by pixelsId will be used with a Projection suffix. For example, GFP-H2B Image of HeLa Cells will have an Image name of GFP-H2B Image of HeLa Cells Projection used for the projection.
        _ctx -- The request context for the invocation.
        Returns: The Id of the newly created Image which has been projected.
        Throws:
        ValidationException -- Where: algorithm is unknown tStart is out of range tEnd is out of range tStart is greater than tEnd channelList is null or has indexes out of range zStart is out of range zEnd is out of range zStart is greater than zEnd the Pixels set qualified by pixelsId is not locatable.
        """
        def projectPixels(self, pixelsId, pixelsType, algorithm, tStart, tEnd, channelList, stepping, zStart, zEnd, name, _ctx=None):
            return _M_omero.api.IProjection._op_projectPixels.invoke(self, ((pixelsId, pixelsType, algorithm, tStart, tEnd, channelList, stepping, zStart, zEnd, name), _ctx))

        """
        Performs a projection through selected optical sections and
        optical sections for a given set of time points of a Pixels
        set. The Image which is linked to the Pixels set will be
        copied using
        {@code omero.api.IPixels.copyAndResizeImage}.
        Arguments:
        pixelsId -- The source Pixels set Id.
        pixelsType -- The destination Pixels type. If null, the source Pixels set pixels type will be used.
        algorithm -- MAXIMUM_INTENSITY, MEAN_INTENSITY or SUM_INTENSITY. NOTE: When performing a SUM_INTENSITY projection, pixel values will be pinned to the maximum pixel value of the destination Pixels type.
        tStart -- Timepoint to start projecting from.
        tEnd -- Timepoint to finish projecting.
        channelList -- List of the channel indexes to use while calculating the projection.
        stepping -- Stepping value to use while calculating the projection. For example, stepping=1 will use every optical section from start to end where stepping=2 will use every other section from start to end to perform the projection.
        zStart -- Optical section to start projecting from.
        zEnd -- Optical section to finish projecting.
        name -- Name for the newly created image. If null the name of the Image linked to the Pixels qualified by pixelsId will be used with a Projection suffix. For example, GFP-H2B Image of HeLa Cells will have an Image name of GFP-H2B Image of HeLa Cells Projection used for the projection.
        _response -- The asynchronous response callback.
        _ex -- The asynchronous exception callback.
        _sent -- The asynchronous sent callback.
        _ctx -- The request context for the invocation.
        Returns: An asynchronous result object for the invocation.
        """
        def begin_projectPixels(self, pixelsId, pixelsType, algorithm, tStart, tEnd, channelList, stepping, zStart, zEnd, name, _response=None, _ex=None, _sent=None, _ctx=None):
            return _M_omero.api.IProjection._op_projectPixels.begin(self, ((pixelsId, pixelsType, algorithm, tStart, tEnd, channelList, stepping, zStart, zEnd, name), _response, _ex, _sent, _ctx))

        """
        Performs a projection through selected optical sections and
        optical sections for a given set of time points of a Pixels
        set. The Image which is linked to the Pixels set will be
        copied using
        {@code omero.api.IPixels.copyAndResizeImage}.
        Arguments:
        pixelsId -- The source Pixels set Id.
        pixelsType -- The destination Pixels type. If null, the source Pixels set pixels type will be used.
        algorithm -- MAXIMUM_INTENSITY, MEAN_INTENSITY or SUM_INTENSITY. NOTE: When performing a SUM_INTENSITY projection, pixel values will be pinned to the maximum pixel value of the destination Pixels type.
        tStart -- Timepoint to start projecting from.
        tEnd -- Timepoint to finish projecting.
        channelList -- List of the channel indexes to use while calculating the projection.
        stepping -- Stepping value to use while calculating the projection. For example, stepping=1 will use every optical section from start to end where stepping=2 will use every other section from start to end to perform the projection.
        zStart -- Optical section to start projecting from.
        zEnd -- Optical section to finish projecting.
        name -- Name for the newly created image. If null the name of the Image linked to the Pixels qualified by pixelsId will be used with a Projection suffix. For example, GFP-H2B Image of HeLa Cells will have an Image name of GFP-H2B Image of HeLa Cells Projection used for the projection.
        Returns: The Id of the newly created Image which has been projected.
        Throws:
        ValidationException -- Where: algorithm is unknown tStart is out of range tEnd is out of range tStart is greater than tEnd channelList is null or has indexes out of range zStart is out of range zEnd is out of range zStart is greater than zEnd the Pixels set qualified by pixelsId is not locatable.
        """
        def end_projectPixels(self, _r):
            return _M_omero.api.IProjection._op_projectPixels.end(self, _r)

        def checkedCast(proxy, facetOrCtx=None, _ctx=None):
            return _M_omero.api.IProjectionPrx.ice_checkedCast(proxy, '::omero::api::IProjection', facetOrCtx, _ctx)
        checkedCast = staticmethod(checkedCast)

        def uncheckedCast(proxy, facet=None):
            return _M_omero.api.IProjectionPrx.ice_uncheckedCast(proxy, facet)
        uncheckedCast = staticmethod(uncheckedCast)

        def ice_staticId():
            return '::omero::api::IProjection'
        ice_staticId = staticmethod(ice_staticId)

    _M_omero.api._t_IProjectionPrx = IcePy.defineProxy('::omero::api::IProjection', IProjectionPrx)

    _M_omero.api._t_IProjection = IcePy.defineClass('::omero::api::IProjection', IProjection, -1, (), True, False, None, (_M_omero.api._t_ServiceInterface,), ())
    IProjection._ice_type = _M_omero.api._t_IProjection

    IProjection._op_projectStack = IcePy.Operation('projectStack', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), IcePy._t_long, False, 0), ((), _M_omero.model._t_PixelsType, False, 0), ((), _M_omero.constants.projection._t_ProjectionType, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0)), (), ((), _M_Ice._t_ByteSeq, False, 0), (_M_omero._t_ServerError,))
    IProjection._op_projectPixels = IcePy.Operation('projectPixels', Ice.OperationMode.Normal, Ice.OperationMode.Normal, True, None, (), (((), IcePy._t_long, False, 0), ((), _M_omero.model._t_PixelsType, False, 0), ((), _M_omero.constants.projection._t_ProjectionType, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), _M_omero.sys._t_IntList, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_int, False, 0), ((), IcePy._t_string, False, 0)), (), ((), IcePy._t_long, False, 0), (_M_omero._t_ServerError,))

    _M_omero.api.IProjection = IProjection
    del IProjection

    _M_omero.api.IProjectionPrx = IProjectionPrx
    del IProjectionPrx

# End of module omero.api

__name__ = 'omero'

# End of module omero
